@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2009-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2009-2015 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -25,6 +25,7 @@ use Cairo;
 
 use List::Util qw(min max sum);
 
+use AMC::Path;
 use AMC::Basic;
 use AMC::Exec;
 use AMC::Gui::Avancement;
@@ -44,6 +45,7 @@ my $fich_bareme='';
 my $id_file='';
 
 my $seuil=0.1;
+my $seuil_up=1.0;
 
 my $data_dir='';
 
@@ -120,6 +122,10 @@ GetOptions("cr=s"=>\$cr_dir,
 
 set_debug($debug);
 
+print (("*"x60)."\n");
+print "* WARNING: AMC-annote is now obsolete\n* Please move to AMC-annotate\n";
+print (("*"x60)."\n");
+
 for(split(/,/,join(',',@o_symbols))) {
     if(/^([01]-[01]):(none|circle|mark|box)(?:\/([\#a-z0-9]+))?$/) {
 	$symboles{$1}={type=>$2,color=>$3};
@@ -165,6 +171,7 @@ my $assoc=$data->module('association');
 my $layout=$data->module('layout');
 
 $seuil=$scoring->variable_transaction('darkness_threshold');
+$seuil_up=$scoring->variable_transaction('darkness_threshold_up');
 
 #################################
 
@@ -366,7 +373,7 @@ print "* Annotation\n";
 
       # ticked on this scan?
       my $cochee=$capture->ticked($p->{'student'},$p->{'copy'},
-				  $q,$r,$seuil);
+				  $q,$r,$seuil,$seuil_up);
 
       debug "Q=$q R=$r $bonne-$cochee";
 
@@ -379,7 +386,7 @@ print "* Annotation\n";
 	}
       }
 
-      if(!($layout->get_box_flags($p->{'student'},$q,$r)
+      if(!($layout->get_box_flags($p->{'student'},$q,$r,BOX_ROLE_ANSWER)
 		  & BOX_FLAGS_DONTANNOTATE)) {
 	if($sy->{type} eq 'circle') {
 	  cercle_coors($context,$b->{'zoneid'},$sy->{color});
