@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -47,10 +47,12 @@ my $sendmail_path='/usr/sbin/sendmail';
 my $smtp_host='smtp';
 my $smtp_port=25;
 my $text='';
+my $text_content_type='text/plain';
 my $subject='';
 my $project_name='';
 my $cc='';
 my $bcc='';
+my $log_file='';
 my @attach_files=();
 
 @ARGV=unpack_args(@ARGV);
@@ -66,6 +68,7 @@ GetOptions("project=s"=>\$project_dir,
 	   "email-column=s"=>\$email_column,
 	   "sender=s"=>\$sender,
 	   "text=s"=>\$text,
+	   "text-content-type:s"=>\$text_content_type,
 	   "subject=s"=>\$subject,
 	   "transport=s"=>\$transport,
 	   "sendmail-path=s"=>\$sendmail_path,
@@ -75,6 +78,7 @@ GetOptions("project=s"=>\$project_dir,
 	   "progression=s"=>\$progress,
 	   "progression-id=s"=>\$progress_id,
 	   "attach=s"=>\@attach_files,
+           "log=s"=>\$log_file,
 	   "cc=s"=>\$cc,
 	   "bcc=s"=>\$bcc,
 	   );
@@ -120,6 +124,12 @@ if(-f $ids_file) {
   debug "IDS file $ids_file not found";
 }
 
+if($log_file) {
+  open(LOGF,">>",$log_file) or
+    debug "Error opening log file $log_file: $!";
+  print LOGF localtime." Starting mailing...\n";
+}
+
 my $avance=AMC::Gui::Avancement::new($progress,'id'=>$progress_id);
 
 my $data=AMC::Data->new($data_dir);
@@ -208,7 +218,7 @@ STUDENT: for my $i (@$r) {
       my @parts=
 	(
 	 Email::MIME->create(attributes=>
-			     {content_type => "text/plain",
+			     {content_type => $text_content_type,
 			      encoding     => "base64",
 			      charset      => "UTF-8",
 			     },
@@ -232,26 +242,31 @@ STUDENT: for my $i (@$r) {
 		);
       $email->header_str_set(Cc=>$cc) if($cc);
 
-      my @all_dests=($dest);
+      my @all_dests=(parse_add($dest));
       push @all_dests,parse_add($cc) if($cc);
       push @all_dests,parse_add($bcc) if($bcc);
       my $b=eval {
 	sendmail($email,{'transport'=>$t,to=>\@all_dests});
       } || $@;
+
+      my $status='OK';
+      my $m='';
       if($b) {
-	my $status='OK';
-	my $m='';
 	if($b->isa('Email::Sender::Failure')) {
 	  $status='FAILED';
 	  $m=$b->message;
 	  $m =~ s/[\n\r]+/ | /g;
 	}
-	print "$status [$i->{'id'}] $m\n";
-	debug "$status [$i->{'id'}] $m";
       } else {
-	debug "sendmail failed";
-	print "FAIL [$i->{'id'}] Email::Sender error\n";
+        $status='FAILED';
+        $m='Email::Sender error';
       }
+      print "$status [$i->{'id'}] $m\n";
+      debug "$status [$i->{'id'}] $m";
+      print LOGF "$status [$i->{'id'} -> $dest] $m\n" if($log_file);
+      $report->report_mailing(@sc,
+                              ($status eq 'OK' ? REPORT_MAIL_OK : REPORT_MAIL_FAILED),
+                              $m,'now');
     } else {
       debug "No file: $file";
     }
@@ -265,3 +280,6 @@ $data->end_transaction('Mail');
 
 $avance->fin();
 
+if($log_file) {
+  close(LOGF);
+}
