@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -63,6 +63,14 @@ my $timestamp=time();
 		'sp'=>65536*72.27,
     );
 
+# association code_in_amc_file => BOX_ROLE_*
+my %role=(
+	  'case'=>BOX_ROLE_ANSWER,
+	  'casequestion'=>BOX_ROLE_QUESTIONONLY,
+	  'score'=>BOX_ROLE_SCORE,
+	  'scorequestion'=>BOX_ROLE_SCOREQUESTION,
+	 );
+
 sub read_inches {
     my ($dim)=@_;
     if($dim =~ /^\s*([0-9]*\.?[0-9]*)\s*([a-zA-Z]+)\s*$/) {
@@ -93,6 +101,7 @@ my @pre_assoc=();
 my $cases;
 my $page_number=0;
 my $i='';
+my %with_vars=();
 
 sub add_flag {
   my ($x,$flag)=@_;
@@ -116,7 +125,7 @@ debug "Reading $src...";
 
 open(SRC,$src) or die "Unable to open $src : $!";
 while(<SRC>) {
-    if(/\\page{([^\}]+)}{([^\}]+)}{([^\}]+)}/) {
+    if(/\\page\{([^\}]+)\}\{([^\}]+)\}\{([^\}]+)\}/) {
 	my $id=$1;
 	my $dx=$2;
 	my $dy=$3;
@@ -126,7 +135,7 @@ while(<SRC>) {
 		     -dim_x=>read_inches($dx),-dim_y=>read_inches($dy),
 		     -cases=>$cases};
     }
-    if(/\\tracepos\{([^\}]+)\}\{([^\}]*)\}\{([^\}]*)\}(?:\{([^\}]*)\})?/) {
+    if(/\\tracepos\{(.+?)\}\{([+-]?[0-9.]+[a-z]*)\}\{([+-]?[0-9.]+[a-z]*)\}(?:\{([a-zA-Z]*)\})?$/) {
 	$i=$1;
 	my $x=read_inches($2);
 	my $y=read_inches($3);
@@ -155,6 +164,9 @@ while(<SRC>) {
     if(/\\association\{([0-9]+)\}\{(.*)\}/) {
       push @pre_assoc,[$1,$2];
     }
+    if(/\\with\{(.+?)=(.*)\}/) {
+      $build_vars{$1}=$2;
+    }
 }
 close(SRC);
 
@@ -173,7 +185,11 @@ my $delta=(@pages ? 1/(1+$#pages) : 0);
 
 $layout->begin_transaction('MeTe');
 $layout->clear_all;
-$capture->variable('annotate_source_change',time());
+$layout->clear_variables('build:%');
+for my $k (keys %build_vars) {
+  $layout->variable("build:$k",$build_vars{$k});
+}
+annotate_source_change($capture);
 
 debug "Pre-association...";
 
@@ -240,11 +256,12 @@ PAGE: for my $p (@pages) {
 	$layout->statement('NEWDigit')
 	  ->execute(@ep,$1,$2,bbox($c->{$k}));
       }
-      if($k=~/case:(.*):([0-9]+),([0-9]+)$/) {
-	my ($name,$q,$a)=($1,$2,$3);
-	$layout->question_name($q,$name);
+      if($k=~/(case|casequestion|score|scorequestion):(.*):([0-9]+),(-?[0-9]+)$/) {
+	my ($type,$name,$q,$a)=($1,$2,$3,$4);
+	$layout->question_name($q,$name) if($name ne '');
 	$layout->statement('NEWBox')
-	  ->execute(@ep,$q,$a,bbox($c->{$k}),$c->{$k}->{'flags'});
+	  ->execute(@ep,$role{$type},
+		    $q,$a,bbox($c->{$k}),$c->{$k}->{'flags'});
       }
     }
 
@@ -254,7 +271,7 @@ PAGE: for my $p (@pages) {
 debug "Flagging questions...";
 
 for my $f (@flags) {
-  $layout->add_question_flag($f->{'student'},$f->{'question'},$f->{'flags'});
+  $layout->add_question_flag($f->{'student'},$f->{'question'},BOX_ROLE_ANSWER,$f->{'flags'});
 }
 
 debug "Ending transaction...";
