@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2008-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2015 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -28,8 +28,10 @@ use AMC::Data;
 use encoding 'utf8';
 
 my $darkness_threshold=0.1;
+my $darkness_threshold_up=1.0;
 
 my $floor_mark='';
+my $null_mark=0;
 my $perfect_mark=20;
 my $ceiling=1;
 my $granularity='0.5';
@@ -38,6 +40,7 @@ my $data_dir='';
 
 my $postcorrect_student='';
 my $postcorrect_copy='';
+my $postcorrect_set_multiple='';
 
 my $progres=1;
 my $progres_id='';
@@ -46,14 +49,17 @@ my $debug='';
 
 GetOptions("data=s"=>\$data_dir,
 	   "seuil=s"=>\$darkness_threshold,
+	   "seuil-up=s"=>\$darkness_threshold_up,
 	   "debug=s"=>\$debug,
 	   "grain=s"=>\$granularity,
 	   "arrondi=s"=>\$rounding_scheme,
 	   "notemax=s"=>\$perfect_mark,
 	   "plafond!"=>\$ceiling,
 	   "notemin=s"=>\$floor_mark,
+	   "notenull=s"=>\$null_mark,
 	   "postcorrect-student=s"=>\$postcorrect_student,
 	   "postcorrect-copy=s"=>\$postcorrect_copy,
+	   "postcorrect-set-multiple!"=>\$postcorrect_set_multiple,
 	   "progression-id=s"=>\$progres_id,
 	   "progression=s"=>\$progres,
 	   );
@@ -62,7 +68,7 @@ set_debug($debug);
 
 # fixes decimal separator ',' potential problem, replacing it with a
 # dot.
-for my $x (\$granularity,\$floor_mark,\$perfect_mark) {
+for my $x (\$granularity,\$null_mark,\$floor_mark,\$perfect_mark) {
     $$x =~ s/,/./;
     $$x =~ s/\s+//;
 }
@@ -126,13 +132,16 @@ my $avance=AMC::Gui::Avancement::new($progres,'id'=>$progres_id);
 my $data=AMC::Data->new($data_dir);
 my $capture=$data->module('capture');
 my $scoring=$data->module('scoring');
+my $layout=$data->module('layout');
 
 # Uses an AMC::Scoring object to actually compute the questions
 # scores.
 
 my $bar=AMC::Scoring::new('onerror'=>'die',
 			  'data'=>$data,
-			  'seuil'=>$darkness_threshold);
+			  'seuil'=>$darkness_threshold,
+			  'seuil_up'=>$darkness_threshold_up,
+			 );
 
 $avance->progres(0.05);
 
@@ -140,13 +149,19 @@ $avance->progres(0.05);
 
 $data->begin_transaction('MARK');
 
+# get some useful build variables
+
+my $code_digit_pattern=$layout->code_digit_pattern();
+
 # Write the variables values in the database, so that they can be
 # retrieved later, and clears all the scores that could have been
 # already computed.
 
-$capture->variable('annotate_source_change',time());
+annotate_source_change($capture);
 $scoring->clear_score;
 $scoring->variable('darkness_threshold',$darkness_threshold);
+$scoring->variable('darkness_threshold_up',$darkness_threshold_up);
+$scoring->variable('mark_null',$null_mark);
 $scoring->variable('mark_floor',$floor_mark);
 $scoring->variable('mark_max',$perfect_mark);
 $scoring->variable('ceiling',$ceiling);
@@ -154,6 +169,7 @@ $scoring->variable('rounding',$rounding_scheme);
 $scoring->variable('granularity',$granularity);
 $scoring->variable('postcorrect_student',$postcorrect_student);
 $scoring->variable('postcorrect_copy',$postcorrect_copy);
+$scoring->variable('postcorrect_set_multiple',$postcorrect_set_multiple);
 
 # Gets the student/copy pairs that has been captured. Each element
 # from the array @captured_studentcopy is an arrayref containing a different
@@ -172,7 +188,8 @@ $delta/=(1+$#captured_studentcopy) if($#captured_studentcopy>=0);
 
 if($postcorrect_student) {
     $scoring->postcorrect($postcorrect_student,$postcorrect_copy,
-			  $darkness_threshold);
+			  $darkness_threshold,$darkness_threshold_up,
+			  $postcorrect_set_multiple);
 }
 
 # Processes each student/copy in turn
@@ -187,7 +204,7 @@ for my $sc (@captured_studentcopy) {
   # Gets the scoring strategy for current student/copy, including
   # which answers are correct, from the scoring database.
 
-  my $ssb=$scoring->student_scoring_base(@$sc,$darkness_threshold);
+  my $ssb=$scoring->student_scoring_base(@$sc,$darkness_threshold,$darkness_threshold_up);
 
   # transmits the main strategy (default strategy options values for
   # all questions) to the scoring engine.
@@ -223,11 +240,11 @@ for my $sc (@captured_studentcopy) {
     ($xx,$why)=$bar->score_question(@$sc,$q,0);
     ($max_score)=$bar->score_max_question($sc->[0],$q);
 
-    # If the title of the question is 'codename.N' (with a numerical
-    # N), then thhis question represents a digit from a AMCcode, so we
+    # If the title of the question is 'codename[N]' (with a numerical
+    # N), then this question represents a digit from a AMCcode, so we
     # collect the value in the %codes hash.
 
-    if ($q->{'title'} =~ /^(.*)\.([0-9]+)$/) {
+    if ($q->{'title'} =~ /^(.*)$code_digit_pattern$/) {
       $codes{$1}->{$2}=$xx;
     }
 
@@ -260,21 +277,24 @@ for my $sc (@captured_studentcopy) {
   my $x;
 
   if ($perfect_mark>0) {
-    $x=$perfect_mark/$granularity*$total/$max_i;
+    $x=($perfect_mark-$null_mark)/$granularity*$total/$max_i;
   } else {
     $x=$total/$granularity;
   }
   $x=&$rounding($x) if($rounding);
   $x*=$granularity;
+  $x+=$null_mark;
 
   # Apply ceiling
 
-  $x=$perfect_mark if($perfect_mark>0 && $ceiling && $x>$perfect_mark);
+  $x=$perfect_mark if($perfect_mark>0 && $ceiling && ($x-$perfect_mark)*($perfect_mark-$null_mark)>0);
 
   # Apply floor
 
   if ($floor_mark ne '' && $floor_mark !~ /[a-z]/i) {
-    $x=$floor_mark if($x<$floor_mark);
+    $x=$floor_mark
+      if(($perfect_mark==0 && $x<$floor_mark) ||
+	 ($x-$floor_mark)*($perfect_mark-$null_mark)<0);
   }
 
   # Writes the student's final mark in the scoring database
