@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -76,6 +76,20 @@ sub path {
   return($self->{'data'}->module_path($self->{'name'}));
 }
 
+# module(name) returns another module from same data
+
+sub module {
+  my ($self,$name)=@_;
+  return($self->{data}->module($name));
+}
+
+# require_module(name) loads the module for the same data
+
+sub require_module {
+  my ($self,$name)=@_;
+  return($self->{data}->require_module($name));
+}
+
 # vacuum() loads the SQLite database separately, and asks for VACUUM
 # on it.
 
@@ -133,8 +147,8 @@ sub sql_quote {
 # elements of @bind.
 
 sub sql_do {
-    my ($self,$sql)=@_;
-    $self->{'data'}->sql_do($sql);
+    my ($self,$sql,@bind)=@_;
+    $self->{'data'}->sql_do($sql,@bind);
 }
 
 # sql_single($sql,@bind) calls the SQL query $sql (SQL string or
@@ -336,13 +350,18 @@ sub variable_transaction {
     }
 }
 
-# clear_variables clear all variables values that are not used
-# internally by the module (keeps the 'version' variable, for
-# exemple).
+# clear_variables($pattern) clear all variables values that are not
+# used internally by the module (keeps the 'version' variable, for
+# exemple). If $pattern is given, only delete variables LIKE $pattern.
 
 sub clear_variables {
-  my ($self)=@_;
-  $self->sql_do("DELETE FROM ".$self->table("variables")." WHERE name != 'version'");
+  my ($self,$pattern)=@_;
+  if($pattern) {
+    $self->sql_do("DELETE FROM ".$self->table("variables")." WHERE name != 'version'"
+		  ." AND name LIKE ?",$pattern);
+  } else {
+    $self->sql_do("DELETE FROM ".$self->table("variables")." WHERE name != 'version'");
+  }
 }
 
 # version_check upgrades the module database to the last version.
