@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -460,27 +460,35 @@ sub define_statements {
       ."   FROM $t_zone"
       ."   WHERE $t_zone.student=$t_page.student"
       ."     AND $t_zone.page=$t_page.page AND $t_zone.copy=$t_page.copy"
-      ."     AND total>0) AS delta"
+      ."     AND $t_zone.type=? AND total>0) AS delta"
+      .",(SELECT MIN(ABS(1.0*black/total-?))"
+      ."   FROM $t_zone"
+      ."   WHERE $t_zone.student=$t_page.student"
+      ."     AND $t_zone.page=$t_page.page AND $t_zone.copy=$t_page.copy"
+      ."     AND $t_zone.type=? AND total>0) AS delta_up"
       ." FROM $t_page"},
      'pages'=>{'sql'=>"SELECT * FROM $t_page"
 	       ." WHERE timestamp_auto>0 OR timestamp_manual>0"},
      'missingPages'=>
      {'sql'=>"SELECT enter.student AS student,enter.page AS page ,$t_page.copy AS copy"
-      ." FROM (SELECT student,page FROM $t_box"
+      ." FROM (SELECT student,page FROM $t_box WHERE role=1"
       ."       UNION SELECT student,page FROM $t_namefield) AS enter,"
       ."      $t_page"
       ." ON enter.student=$t_page.student"
       ." EXCEPT SELECT student,page,copy FROM $t_page"
       ." ORDER BY student,copy,page"},
+     'questionOnlyPages'=>
+     {'sql'=>"SELECT student,page,0 AS copy FROM $t_box WHERE role=2"
+      ." EXCEPT SELECT student,page,0 AS copy FROM $t_box WHERE role=1"},
      'noCapturePages'=>
-     {'sql'=>"SELECT student,page,0 AS copy FROM $t_box"
+     {'sql'=>"SELECT student,page,0 AS copy FROM $t_box WHERE role=1"
       ." UNION SELECT student,page,0 AS copy FROM $t_namefield"
       ." EXCEPT SELECT student,page,copy FROM $t_page"
       ." WHERE timestamp_auto>0 OR timestamp_manual>0"},
      'pageNearRatio'=>{'sql'=>"SELECT MIN(ABS(1.0*black/total-?))"
 		       ." FROM $t_zone"
 		       ." WHERE student=? AND page=? AND copy=? AND total>0"},
-     'pageZones'=>{'sql'=>"SELECT zoneid,id_a,id_b,total,black,manual FROM $t_zone"
+     'pageZones'=>{'sql'=>"SELECT zoneid,id_a,id_b,total,black,manual,1 AS role FROM $t_zone"
 		   ." WHERE student=? AND page=? AND copy=? AND type=?"},
      'zonesImages'=>{'sql'=>"SELECT image FROM $t_zone"
 		     ." WHERE student=? AND page=? AND copy=? AND type=?"},
@@ -518,7 +526,7 @@ sub define_statements {
      'ticked'=>{'sql'=>"SELECT CASE"
 		." WHEN manual >= 0 THEN manual"
 		." WHEN total<=0 THEN -1"
-		." WHEN black >= ? * total THEN 1"
+		." WHEN black >= ? * total AND black <= ? * total THEN 1"
 		." ELSE 0"
 		." END FROM $t_zone"
 		." WHERE student=? AND copy=? AND type=? AND id_a=? AND id_b=?"},
@@ -528,7 +536,7 @@ sub define_statements {
 		    ." WHEN why=\"E\" THEN 0"
 		    ." WHEN zone.manual >= 0 THEN zone.manual"
 		    ." WHEN zone.total<=0 THEN -1"
-		    ." WHEN zone.black >= ? * zone.total THEN 1"
+		    ." WHEN zone.black >= ? * zone.total AND zone.black <= ? * zone.total THEN 1"
 		    ." ELSE 0"
 		    ." END) AS nb"
 		    ." FROM $t_zone AS zone, scoring.scoring_score AS score"
@@ -552,7 +560,7 @@ sub define_statements {
      'tickedList'=>{'sql'=>"SELECT CASE"
 		    ." WHEN manual >= 0 THEN manual"
 		    ." WHEN total<=0 THEN -1"
-		    ." WHEN black >= ? * total THEN 1"
+		    ." WHEN black >= ? * total AND black <= ? * total THEN 1"
 		    ." ELSE 0"
 		    ." END FROM $t_zone"
 		    ." WHERE student=? AND copy=? AND type=? AND id_a=?"
@@ -560,7 +568,7 @@ sub define_statements {
      'tickedPage'=>{'sql'=>"SELECT CASE"
 		    ." WHEN manual >= 0 THEN manual"
 		    ." WHEN total<=0 THEN -1"
-		    ." WHEN black >= ? * total THEN 1"
+		    ." WHEN black >= ? * total AND black <= ? * total THEN 1"
 		    ." ELSE 0"
 		    ." END,id_a,id_b FROM $t_zone"
 		    ." WHERE student=? AND page=? AND copy=? AND type=?"},
@@ -825,20 +833,46 @@ sub n_pages_auto {
   return($self->sql_list($self->statement('nPagesAuto')));
 }
 
-# page_sensitivity($student,$page,$copy,$threshold) returns a
-# sensitivity value for the page automated data capture. When this
-# value is low (near 0), black and white boxes have darkness values
-# far away from the threshold, and thus one can think that it is well
-# recognized if the boxes are ticked or not. With high values of the
-# sensitivity (the maximal value is 10), ticked boxes are not robustly
-# detected: if one changes a little the darkness threshold, boxes can
-# pass from the ticked to not-ticked state or vice versa.
+# page_sensitivity($student,$page,$copy,$darkness_threshold,$darkness_threshold_up)
+# returns a sensitivity value for the page automated data
+# capture. When this value is low (near 0), black and white boxes have
+# darkness values far away from the threshold, and thus one can think
+# that it is well recognized if the boxes are ticked or not. With high
+# values of the sensitivity (the maximal value is 10), ticked boxes
+# are not robustly detected: if one changes a little the darkness
+# threshold, boxes can pass from the ticked to not-ticked state or
+# vice versa.
+
+sub sensitivity_down {
+  my ($delta,$threshold)=@_;
+  return(defined($delta) && $threshold>0 ? 10*($threshold-$delta)/$threshold : undef);
+}
+
+sub sensitivity_up {
+  my ($delta,$threshold)=@_;
+  return(defined($delta) && $threshold<1 ? sensitivity($delta,1-$threshold) : undef);
+}
+
+sub sensitivity {
+  my ($delta,$threshold,$delta_up,$threshold_up)=@_;
+  my $s=sensitivity_down($delta,$threshold);
+  my $s_up=sensitivity_up($delta_up,$threshold_up);
+  return(defined($s_up) && (!defined($s) || $s_up>$s) ?
+	 $s_up : $s);
+}
 
 sub page_sensitivity {
-  my ($self,$student,$page,$copy,$threshold)=@_;
+  my ($self,$student,$page,$copy,$darkness_threshold,$darkness_threshold_up)=@_;
+  die "Missing parameters in pqge_sensitivity call"
+    if(!defined($darkness_threshold_up));
   my $delta=$self->sql_single($self->statement('pageNearRatio'),
-			      $threshold,$student,$page,$copy);
-  return(defined($delta) ? 10*($threshold-$delta)/$threshold : undef);
+			      $darkness_threshold,
+			      $student,$page,$copy);
+  my $delta_up=$self->sql_single($self->statement('pageNearRatio'),
+				 $darkness_threshold_up,
+				 $student,$page,$copy);
+  return(sensitivity($delta,$darkness-threshold,
+		     $delta_up,$darkness_threshold_up));
 }
 
 # page_summary($student,$page,$copy,%options) returns a hash %s giving
@@ -862,6 +896,9 @@ sub page_sensitivity {
 #
 # $s{'sensitivity'} is 'red' is the sensitivity exceeds
 # $options{'sensitivity_threshold'}, undef otherwise.
+#
+# $s{'why'} (only available if $options{'why'} is true) collects all
+# 'why' attributes from all qeustions on the page (from the scoring table)
 
 # summaries returns a reference to an array containing the summaries
 # of all pages.
@@ -874,15 +911,24 @@ sub compute_summaries {
 			sprintf($p->{'timestamp_manual'}>0 ? "(%.01f)" : "%.01f",
 				$p->{'mse'})
 			: "---");
-    $p->{'sensitivity'}=(defined($p->{'delta'})
-			 ? 10*($oo{'darkness_threshold'}-$p->{'delta'})
-			 /$oo{'darkness_threshold'} : undef);
+    $p->{'sensitivity'}=sensitivity($p->{'delta'},$oo{'darkness_threshold'},
+				    $p->{'delta_up'},$oo{'darkness_threshold_up'});
     $p->{'sensitivity_string'}=(defined($p->{'sensitivity'}) 
 				? sprintf("%.1f",$p->{'sensitivity'}) : "---");
     $p->{'sensitivity_color'}=(defined($p->{'sensitivity'}) ?
 			       ($p->{'sensitivity'} > $oo{'sensitivity_threshold'}
 				? 'red' : undef) : undef);
   }
+  if($oo{why}) {
+    $self->require_module("layout");
+    my %why=();
+    for my $w ($self->module("scoring")->pages_why()) {
+      $why{pageids_string($w->{student},$w->{page},$w->{copy})}=$w->{why};
+    }
+    for my $p (@$r) {
+      $p->{why}=$why{pageids_string($p->{student},$p->{page},$p->{copy})};
+    }
+  }
   return($r);
 }
 
@@ -892,7 +938,8 @@ sub page_summary {
 				       {Slice=>{}},
 				       $oo{'mse_threshold'},'red',undef,
 				       'lightblue','lightgreen',undef,
-				       $oo{'darkness_threshold'},
+				       $oo{'darkness_threshold'},ZONE_BOX,
+				       $oo{'darkness_threshold_up'},ZONE_BOX,
 				       $student,$page,$copy,
 				      );
   if($r && $r->[0]) {
@@ -909,7 +956,8 @@ sub summaries {
 				       {Slice=>{}},
 				       $oo{'mse_threshold'},'red',undef,
 				       'lightblue','lightgreen',undef,
-				       $oo{'darkness_threshold'},
+				       $oo{'darkness_threshold'},ZONE_BOX,
+				       $oo{'darkness_threshold_up'},ZONE_BOX,
 				      );
   return($self->compute_summaries($r,%oo));
 }
@@ -924,19 +972,24 @@ sub zone_darkness {
 			   $zoneid));
 }
 
-# ticked($student,$copy,$question,$answer,$darkness_threshold) returns
-# 1 if the darkness of a particular zone exceeds $darkness_threshold,
-# and 0 otherwise. If a manual data capture occured for this zone, the
+# ticked($student,$copy,$question,$answer,$darkness_threshold,$darkness_threshold_up)
+# returns 1 if the darkness of a particular zone exceeds
+# $darkness_threshold and is less than $darkness_threshold_up, and 0
+# otherwise. If a manual data capture occured for this zone, the
 # darkness is not considered and the manual result is given instead.
 
 sub ticked {
-  my ($self,$student,$copy,$question,$answer,$darkness_threshold)=@_;
-  return($self->sql_single($self->statement('ticked'),$darkness_threshold,
+  my ($self,$student,$copy,$question,$answer,
+      $darkness_threshold,$darkness_threshold_up)=@_;
+  die "Missing parameters in ticked call"
+    if(!defined($darkness_threshold_up));
+  return($self->sql_single($self->statement('ticked'),
+			   $darkness_threshold,$darkness_threshold_up,
 			   $student,$copy,ZONE_BOX,$question,$answer));
 }
 
-# ticked_sums($darkness_threshold) returns a ref to a list of hashrefs
-# like
+# ticked_sums($darkness_threshold,$darkness_threshold_up) returns a
+# ref to a list of hashrefs like
 #
 # [{'question=>1,'answer'=>1,nb=>4},
 #  {'question=>1,'answer'=>'invalid',nb=>1},
@@ -948,10 +1001,12 @@ sub ticked {
 # replied, and where this question got invalid answers.
 
 sub ticked_sums {
-  my ($self,$darkness_threshold)=@_;
+  my ($self,$darkness_threshold,$darkness_threshold_up)=@_;
+  die "Missing parameters in ticked_sums call"
+    if(!defined($darkness_threshold_up));
   return($self->dbh->selectall_arrayref($self->statement('tickedSums'),
 					{ Slice=>{} },
-					$darkness_threshold,ZONE_BOX)
+					$darkness_threshold,$darkness_threshold_up,ZONE_BOX)
 	 );
 }
 
@@ -964,15 +1019,18 @@ sub has_answer_zero {
 			   $student,$copy,ZONE_BOX,$question));
 }
 
-# ticked_list($student,$copy,$question,$darkness_threshold) returns a
-# list with the ticked results for all the answers boxes from a
-# particular question. Answers are ordered with the answer number, so
-# that the answer "None of the above" (if present), with answer number
-# 0, is placed at the beginning.
+# ticked_list($student,$copy,$question,$darkness_threshold,$darkness_threshold_up)
+# returns a list with the ticked results for all the answers boxes
+# from a particular question. Answers are ordered with the answer
+# number, so that the answer "None of the above" (if present), with
+# answer number 0, is placed at the beginning.
 
 sub ticked_list {
-  my ($self,$student,$copy,$question,$darkness_threshold)=@_;
-  return($self->sql_list($self->statement('tickedList'),$darkness_threshold,
+  my ($self,$student,$copy,$question,$darkness_threshold,$darkness_threshold_up)=@_;
+  die "Missing parameters in ticked_list call"
+    if(!defined($darkness_threshold_up));
+  return($self->sql_list($self->statement('tickedList'),
+			 $darkness_threshold,$darkness_threshold_up,
 			 $student,$copy,ZONE_BOX,$question));
 }
 
@@ -981,8 +1039,10 @@ sub ticked_list {
 # list.
 
 sub ticked_list_0 {
-  my ($self,$student,$copy,$question,$darkness_threshold)=@_;
-  my @tl=$self->ticked_list($student,$copy,$question,$darkness_threshold);
+  my ($self,$student,$copy,$question,$darkness_threshold,$darkness_threshold_up)=@_;
+  die "Missing parameters in ticked_list_0 call"
+    if(!defined($darkness_threshold_up));
+  my @tl=$self->ticked_list($student,$copy,$question,$darkness_threshold,$darkness_threshold_up);
   if($self->has_answer_zero($student,$copy,$question)) {
     my $zero=shift @tl;
     push @tl,$zero;
@@ -1246,6 +1306,16 @@ sub no_capture_pages {
   return($self->dbh->selectall_arrayref($self->statement('noCapturePages')));
 }
 
+# no_capture_pages returns a reference to an array of elements
+# [student,page,copy] for pages from the question paper that has boxes
+# from the question but no boxes from the answer sheet (all with
+# copy=0). The result is empty if not in separate answer sheet mode.
+
+sub question_only_pages {
+  my ($self)=@_;
+  return($self->dbh->selectall_arrayref($self->statement('questionOnlyPages')));
+}
+
 # delete_page_data($student,$page,$copy) deletes all data concerning
 # the given page (automatic and manual data capture) from the database.
 
