@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -66,10 +66,13 @@ package AMC::DataModule::layout;
 #
 # * xmin,xmax,ymin,ymax give the box around the name field
 #
-# layout_box lists all the boxes to be ticked on all the pages:
+# layout_box lists all the boxes to be ticked (and other
+# question/answer-related zones in the subject) on all the pages:
 #
 # * student,page identifies the page
 #
+# * role determines the role of this box (see BOX_ROLE_* constants below)
+#
 # * question is the question number. This is NOT the question number
 #   that is printed on the question paper, but an internal question
 #   number associated with question identifier from the LaTeX file
@@ -126,10 +129,16 @@ use constant {
   BOX_FLAGS_DONTSCAN => 0x1,
   BOX_FLAGS_DONTANNOTATE => 0x2,
   BOX_FLAGS_SHAPE_OVAL => 0x10,
+
+  # Do not change these values as they are hard-coded elsewhere
+  BOX_ROLE_ANSWER => 1, # Boxes to be ticked by the student
+  BOX_ROLE_QUESTIONONLY => 2, # In separate answer sheet mode, boxes in the question section
+  BOX_ROLE_SCORE => 100, # Zones to write scores when annotating answer sheets
+  BOX_ROLE_SCOREQUESTION => 101, # The same but for question (in separate answer sheet mode)
 };
 
-our @EXPORT_OK = qw(BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE BOX_FLAGS_SHAPE_OVAL);
-our %EXPORT_TAGS = ( 'flags' => [ qw/BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE BOX_FLAGS_SHAPE_OVAL/ ],
+our @EXPORT_OK = qw(BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE BOX_FLAGS_SHAPE_OVAL BOX_ROLE_ANSWER BOX_ROLE_QUESTIONONLY BOX_ROLE_SCORE BOX_ROLE_SCOREQUESTION);
+our %EXPORT_TAGS = ( 'flags' => [ qw/BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE BOX_FLAGS_SHAPE_OVAL BOX_ROLE_ANSWER BOX_ROLE_QUESTIONONLY BOX_ROLE_SCORE BOX_ROLE_SCOREQUESTION/ ],
 		     );
 
 use AMC::Basic;
@@ -139,7 +148,24 @@ use XML::Simple;
 @ISA=("AMC::DataModule");
 
 sub version_current {
-  return(4);
+  return(5);
+}
+
+sub drop_box_table {
+  my ($self)=@_;
+  $self->sql_do("DROP INDEX ".$self->index("index_box_studentpage"));
+  $self->sql_do("DROP TABLE ".$self->table("box"));
+}
+
+sub create_box_table {
+  my ($self,$tmp)=@_;
+  $self->sql_do("CREATE ".($tmp ? "TEMPORARY ":"")
+		."TABLE IF NOT EXISTS ".($tmp ? "box_tmp" :$self->table("box"))
+		." (student INTEGER, page INTEGER, role INTEGER DEFAULT 1, question INTEGER, answer INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL, flags INTEGER DEFAULT 0, PRIMARY KEY (student,role,question,answer))");
+  if(!$tmp) {
+    $self->sql_do("CREATE INDEX ".$self->index("index_box_studentpage")." ON "
+		  .$self->table("box","self")." (student,page,role)");
+  }
 }
 
 sub version_upgrade {
@@ -158,10 +184,7 @@ sub version_upgrade {
 		      ." (student INTEGER, page INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL)");
 	$self->sql_do("CREATE INDEX ".$self->index("index_namefield")." ON "
 		      .$self->table("namefield","self")." (student,page)");
-	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("box")
-		      ." (student INTEGER, page INTEGER, question INTEGER, answer INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL, flags INTEGER DEFAULT 0, PRIMARY KEY (student,question,answer))");
-	$self->sql_do("CREATE INDEX ".$self->index("index_box_studentpage")." ON "
-		      .$self->table("box","self")." (student,page)");
+	$self->create_box_table;
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("digit")
 		      ." (student INTEGER, page INTEGER, numberid INTEGER, digitid INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL, PRIMARY KEY(student,page,numberid,digitid))");
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("source")
@@ -172,7 +195,7 @@ sub version_upgrade {
 		      ." (student INTEGER PRIMARY KEY, id TEXT)");
 	$self->populate_from_xml;
 
-	return(4);
+	return(5);
     }
     if($old_version==1) {
       $self->sql_do("ALTER TABLE ".$self->table("box")
@@ -213,6 +236,20 @@ sub version_upgrade {
       $self->progression('end');
       return(4);
     }
+    if($old_version==4) {
+      # To change the box table columns, primary key and index, use a
+      # temporary table to transfer all rows
+      $self->create_box_table(1);
+      $self->sql_do("INSERT INTO box_tmp (student,page,question,answer,xmin,xmax,ymin,ymax,flags)"
+		   ." SELECT student,page,question,answer,xmin,xmax,ymin,ymax,flags FROM "
+		   .$self->table("box"));
+      $self->drop_box_table;
+      $self->create_box_table;
+      $self->sql_do("INSERT INTO ".$self->table("box")
+		    ." SELECT * FROM box_tmp");
+      $self->sql_do("DROP TABLE box_tmp");
+      return(5);
+    }
     return('');
 }
 
@@ -258,7 +295,7 @@ sub populate_from_xml {
 		    }
 		    for my $c (@{$l->{'case'}}) {
 			$self->statement('NEWBox')->execute(
-			    @lid,(map { $c->{$_} } (qw/question reponse xmin xmax ymin ymax/)),0
+			    @lid,BOX_ROLE_ANSWER,(map { $c->{$_} } (qw/question reponse xmin xmax ymin ymax/)),0
 			    );
 		    }
 		    for my $d (@{$l->{'chiffre'}}) {
@@ -314,8 +351,8 @@ sub define_statements {
        'NEWMark'=>{'sql'=>"INSERT INTO ".$self->table("mark")
 		   ." (student,page,corner,x,y) VALUES (?,?,?,?,?)"},
        'NEWBox'=>{'sql'=>"INSERT INTO ".$self->table("box")
-		  ." (student,page,question,answer,xmin,xmax,ymin,ymax,flags)"
-		  ." VALUES (?,?,?,?,?,?,?,?,?)"},
+		  ." (student,page,role,question,answer,xmin,xmax,ymin,ymax,flags)"
+		  ." VALUES (?,?,?,?,?,?,?,?,?,?)"},
        'NEWDigit'=>{'sql'=>"INSERT INTO ".$self->table("digit")
 		    ." (student,page,numberid,digitid,xmin,xmax,ymin,ymax)"
 		    ." VALUES (?,?,?,?,?,?,?,?)"},
@@ -334,36 +371,42 @@ sub define_statements {
 			     ." WHERE student=? ORDER BY page"},
        'STUDENTS'=>{'sql'=>"SELECT student FROM ".$self->table("page")
 		    ." GROUP BY student ORDER BY student"},
+       'pageQuestionBoxes'=>
+       {'sql'=>"SELECT question AS id_a,answer AS id_b,role"
+	." FROM ".$self->table("box")
+	." WHERE role=2 AND student=? AND page=?"},
        'Q_Flag'=>{'sql'=>"UPDATE ".$self->table("box")
-		  ." SET flags=flags|? WHERE student=? AND question=?"},
+		  ." SET flags=flags|? WHERE student=? AND question=? AND role=?"},
        'A_Flags'=>{'sql'=>"SELECT flags FROM ".$self->table("box")
-		  ." WHERE student=? AND question=? AND answer=?"},
+		  ." WHERE student=? AND question=? AND answer=? AND role=?"},
+       'A_All'=>{'sql'=>"SELECT * FROM ".$self->table("box")
+		 ." WHERE student=? AND question=? AND answer=? AND role=?"},
        'PAGES_STUDENT_box'=>{'sql'=>"SELECT page FROM ".$self->table("box")
-			     ." WHERE student=? GROUP BY student,page"},
+			     ." WHERE student=? AND role=? GROUP BY student,page"},
        'PAGES_STUDENT_namefield'=>{'sql'=>"SELECT page FROM ".$self->table("namefield")
 				   ." WHERE student=? GROUP BY student,page"},
        'PAGES_STUDENT_enter'=>
        {'sql'=>"SELECT page FROM ("
-	."SELECT student,page FROM ".$self->table("box")." UNION "
+	."SELECT student,page FROM ".$self->table("box")." WHERE role=1 UNION "
 	."SELECT student,page FROM ".$self->table("namefield")
 	.") AS enter WHERE student=? GROUP BY student,page"},
        'PAGES_enter'=>
        {'sql'=>"SELECT student,page FROM ("
-	."SELECT student,page FROM ".$self->table("box")." UNION "
+	."SELECT student,page FROM ".$self->table("box")." WHERE role=1 UNION "
 	."SELECT student,page FROM ".$self->table("namefield")
 	.") AS enter GROUP BY student,page ORDER BY student,page"},
        'MAX_enter'=>
        {'sql'=>"SELECT MAX(n) FROM"
 	." ( SELECT COUNT(*) AS n FROM"
-	."   ( SELECT student,page FROM ".$self->table("box")
+	."   ( SELECT student,page FROM ".$self->table("box")." WHERE role=1"
 	."     UNION SELECT student,page FROM ".$self->table("namefield")
 	."   ) GROUP BY student )"},
        'DEFECT_NO_BOX'=>
        {'sql'=>"SELECT student FROM (SELECT student FROM ".$self->table("page")
 	." GROUP BY student) AS list"
 	." WHERE student>0 AND"
-	."   NOT EXISTS(SELECT * FROM ".$self->table("box")." AS local"
-	."              WHERE local.student=list.student)"},
+	."   NOT EXISTS(SELECT * FROM ".$self->table("box")." AS local WHERE role=1 AND"
+	."              local.student=list.student)"},
        'DEFECT_NO_NAME'=>
        {'sql'=>"SELECT student FROM (SELECT student FROM ".$self->table("page")
 	." GROUP BY student) AS list"
@@ -385,7 +428,7 @@ sub define_statements {
        'subjectpageForStudentA'=>
        {'sql'=>"SELECT MIN(p.subjectpage),MAX(p.subjectpage) FROM "
 	.$self->table("page")." AS p"
-	." ,( SELECT student,page FROM ".$self->table("box")
+	." ,( SELECT student,page FROM ".$self->table("box")." WHERE role=1"
 	."    UNION"
 	."    SELECT student,page FROM ".$self->table("namefield")." ) AS a"
 	." ON p.student=a.student AND p.page=a.page"
@@ -393,19 +436,24 @@ sub define_statements {
        'studentPage'=>{'sql'=>"SELECT student,page FROM ".$self->table("page")
 		       ." WHERE markdiameter>0"
 		       ." LIMIT 1"},
-       'dims'=>{'sql'=>"SELECT width,height,markdiameter FROM "
+       'dims'=>{'sql'=>"SELECT width,height,markdiameter,dpi FROM "
 		.$self->table("page")
 		." WHERE student=? AND page=?"},
        'mark'=>{'sql'=>"SELECT x,y FROM ".$self->table("mark")
 		." WHERE student=? AND page=? AND corner=?"},
        'pageInfo'=>{'sql'=>"SELECT * FROM ".$self->table("page")
 		    ." WHERE student=? AND page=?"},
+       'studentPageInfo'=>{'sql'=>"SELECT * FROM ".$self->table("page")
+			   ." WHERE student=? ORDER BY page"},
        'digitInfo'=>{'sql'=>"SELECT * FROM ".$self->table("digit")
 		     ." WHERE student=? AND page=?"},
        'boxInfo'=>{'sql'=>"SELECT * FROM ".$self->table("box")
-		   ." WHERE student=? AND page=?"},
+		   ." WHERE student=? AND page=? AND role>=? AND role<=?"},
        'namefieldInfo'=>{'sql'=>"SELECT * FROM ".$self->table("namefield")
 			 ." WHERE student=? AND page=?"},
+       'scoreZones'=>{'sql'=>"SELECT * FROM ".$self->table("box")
+		      ." WHERE student=? AND page=? AND question=?"
+		      ." AND role>=? AND role<=?"},
        'exists'=>{'sql'=>"SELECT COUNT(*) FROM ".$self->table("page")
 		  ." WHERE student=? AND page=? AND checksum=?"},
        'questionName'=>{'sql'=>"SELECT name FROM ".$self->table("question")
@@ -445,6 +493,10 @@ sub define_statements {
 		       ."         AS ratio"
 		       ."  FROM ".$self->table("page")
 		       ." )"},
+       'MapQuestionPage'=>
+       {'sql'=>"SELECT student, page, question "
+	." FROM ".$self->table("box")." WHERE answer=1"
+       },
       };
 }
 
@@ -525,6 +577,14 @@ sub full_ids {
     return($self->sql_list($self->statement('FULLIDS')));
 }
 
+#Return list of student, page, question
+sub student_question_page{
+    my ($self)=@_;
+    my @list = @{$self->dbh
+		     ->selectall_arrayref($self->statement('MapQuestionPage'))};
+    return(@list);
+}
+
 # page_info returns a HASH reference containing all fields in the
 # layout_page row corresponding to the student,page page.
 
@@ -534,16 +594,47 @@ sub page_info {
 	       $self->statement('pageInfo'),{},$student,$page));
 }
 
-# type_info($type,$student,$page) returns an array of HASH references
-# containing all fiels in the $type table ($type may equal digit, box
-# or namefield) corresponding to the $student,$page page.
+# type_info($type,$student,$page,$role) returns an array of HASH
+# references containing all fiels in the $type table ($type may equal
+# digit, box or namefield) corresponding to the $student,$page
+# page. The $role argument (which defaults to BOX_ROLE_ANSWER) is only
+# needed when $type is 'box' (or you can use $type='questionbox' with
+# no $role).
 
 sub type_info {
-  my ($self,$type,$student,$page)=@_;
+  my ($self,$type,$student,$page,$role,$rolemax)=@_;
+  my @args=($student,$page);
+  if($type eq 'questionbox') {
+    $type='box';
+    $role=BOX_ROLE_QUESTIONONLY;
+  }
+  if($type eq 'scorezone') {
+    $type='box';
+    $role=BOX_ROLE_SCORE;
+    $rolemax=BOX_ROLE_SCOREQUESTION;
+  }
+  if($type eq 'box') {
+    $role=BOX_ROLE_ANSWER if(!$role);
+    $rolemax=$role if(!$rolemax);
+    push @args,$role,$rolemax;
+  }
   return(@{$self->dbh
 	     ->selectall_arrayref($self->statement($type.'Info'),
 				  {Slice=>{}},
-				  $student,$page)});
+				  @args)});
+}
+
+# score_zones($student,$page,$question) returns an array of HASH
+# references containing all the rows in the box table corresponding to
+# the given parameters.
+
+sub score_zones {
+  my ($self,$student,$page,$question)=@_;
+  return(@{$self->dbh
+	     ->selectall_arrayref($self->statement('scoreZones'),
+				  {Slice=>{}},
+				  $student,$page,$question,
+				  BOX_ROLE_SCORE,BOX_ROLE_SCOREQUESTION)});
 }
 
 # pages_for_student($student,[%options]) returns a list of the page
@@ -558,8 +649,34 @@ sub type_info {
 sub pages_for_student {
     my ($self,$student,%oo)=@_;
     $oo{'select'}='all' if(!$oo{'select'});
+    @args=($student);
+    if($oo{select} eq 'box') {
+      $oo{role}=BOX_ROLE_ANSWER if(!$oo{role});
+      push @args,$oo{role};
+    }
     return($self->sql_list($self->statement('PAGES_STUDENT_'.$oo{'select'}),
-			   $student));
+			   @args));
+}
+
+# pages_info_for_student($student,[%options]) returns a list of
+# hashrefs with all data from the table page for each page concerning
+# student $student.  With option enter_tag=>1, adds enter=>1 for each
+# page where something has to be entered by the student (boxes or
+# namefield).
+
+sub pages_info_for_student {
+  my ($self,$student,%oo)=@_;
+  my $r=$self->dbh
+    ->selectall_arrayref($self->statement('studentPageInfo'),
+			 {Slice=>{}},
+			 $student);
+  if($oo{enter_tag}) {
+    my %enter_pages=map { $_=>1 } ($self->pages_for_student($student,select=>'enter',role=>$oo{role}));
+    for my $p (@$r) {
+      $p->{enter}=1 if($enter_pages{$p->{page}});
+    }
+  }
+  return(@$r);
 }
 
 # students returns the list of the students numbers.
@@ -639,7 +756,7 @@ sub question_name {
 
 sub clear_all {
     my ($self)=@_;
-    for my $t (qw/page mark namefield box digit source association/) {
+    for my $t (qw/page mark namefield box digit source question association/) {
 	$self->sql_do("DELETE FROM ".$self->table($t));
     }
 }
@@ -690,17 +807,26 @@ sub max_enter {
 # answers boxes for a particular student and question.
 
 sub add_question_flag {
-  my ($self,$student,$question,$flag)=@_;
-  $self->statement('Q_Flag')->execute($flag,$student,$question);
+  my ($self,$student,$question,$role,$flag)=@_;
+  $self->statement('Q_Flag')->execute($flag,$student,$question,$role);
 }
 
-# get_box_glags($student,$question,$answer) returns the flags for the
+# get_box_flags($student,$question,$answer) returns the flags for the
 # corresponding box.
 
 sub get_box_flags {
-  my ($self,$student,$question,$answer)=@_;
+  my ($self,$student,$question,$answer,$role)=@_;
   return($self->sql_single($self->statement('A_Flags'),
-			   $student,$question,$answer));
+			   $student,$question,$answer,$role));
+}
+
+# get_box_info($student,$question,$answer) returns all data from table
+# box for the corresponding box.
+
+sub get_box_info {
+  my ($self,$student,$question,$answer,$role)=@_;
+  return($self->dbh->selectrow_hashref($self->statement('A_All'),{},
+				       $student,$question,$answer,$role));
 }
 
 # new_association($student,$id) adds an association to the
@@ -739,4 +865,20 @@ sub orientation {
   }
 }
 
+# code_digit_pattern() return the regular expression that can be used to
+# detect a code digit.
+
+sub code_digit_pattern {
+  my ($self)=@_;
+  my $type=$self->variable("build:codedigit");
+  if($type eq 'squarebrackets') {
+    # 'codename[N]'
+    return("\\[(\\d+)\\]");
+  } else {
+    # with older AMC version, look for 'codename.N' instead of
+    # 'codename[N]'
+    return("\\.(\\d+)");
+  }
+}
+
 1;
