@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -85,7 +85,7 @@ package AMC::DataModule::scoring;
 # * question is the question number.
 #
 # * type is the question type, either QUESTION_SIMPLE or
-#   QUESTION_MULTIPLE
+#   QUESTION_MULT
 #
 # * indicative is 1 if the question is indicative (the score is not
 #   taken into account when computing the student mark).
@@ -163,10 +163,12 @@ package AMC::DataModule::scoring;
 # postcorrect_student
 # postcorrect_copy    identify the sheet completed by the teacher.
 #
-# --- the folleing values are supplied in the Preferences window
+# postcorrect_set_multiple (see postcorrect function)
 #
-# darkness_threshold is the parameter used for determining wether a
-# box is ticked or not.
+# --- the following values are supplied in the Preferences window
+#
+# darkness_threshold and darkness_threshold_up are the parameters used
+# for determining wether a box is ticked or not.
 #
 # mark_floor is the minimum mark to be given to a student.
 #
@@ -194,6 +196,7 @@ our %EXPORT_TAGS = ( 'question' => [ qw/QUESTION_SIMPLE QUESTION_MULT/ ],
 use AMC::Basic;
 use AMC::DataModule;
 use AMC::DataModule::capture ':zone';
+use AMC::DataModule::layout ':flags';
 
 use XML::Simple;
 
@@ -307,6 +310,7 @@ sub populate_from_xml {
   $xml=XMLin($scoring_file,ForceArray => 1,KeyAttr=> [ 'id' ]);
 
   $self->variable('darkness_threshold',$xml->{'seuil'});
+  $self->variable('darkness_threshold_up',1.0);
   $self->variable('mark_floor',$xml->{'notemin'});
   $self->variable('mark_max',$xml->{'notemax'});
   $self->variable('ceiling',$xml->{'plafond'});
@@ -382,7 +386,7 @@ sub define_statements {
      'NEWQuestion'=>{'sql'=>"INSERT OR REPLACE INTO ".$self->table("question")
 		     ." (student,question,type,indicative,strategy)"
 		     ." VALUES (?,?,?,?,?)"},
-     'NEWAnswer'=>{'sql'=>"INSERT INTO ".$self->table("answer")
+     'NEWAnswer'=>{'sql'=>"INSERT OR REPLACE INTO ".$self->table("answer")
 		   ." (student,question,answer,correct,strategy)"
 		   ." VALUES (?,?,?,?,?)"},
      'setAnswerStrat'=>{'sql'=>"UPDATE ".$self->table("answer")
@@ -400,25 +404,22 @@ sub define_statements {
 			." SELECT id_a,id_b,CASE"
 			."   WHEN manual >= 0 THEN manual"
 			."   WHEN total<=0 THEN -1"
-			."   WHEN black >= ? * total THEN 1"
+			."   WHEN black >= ? * total AND black <= ? * total THEN 1"
 			."   ELSE 0"
 			." END"
 			." FROM ".$self->table("zone","capture")
 			." WHERE capture_zone.student=? AND capture_zone.copy=? AND capture_zone.type=?"},
+     'postCorrectMul'=>{'sql'=>"UPDATE ".$self->table("question")
+			." SET type=CASE"
+			."   WHEN (SELECT sum(c) FROM pc_temp"
+			."          WHERE q=question)>1"
+			."   THEN ?"
+			."   ELSE ?"
+			." END"
+		       },
      'postCorrectSet'=>{'sql'=>"UPDATE ".$self->table("answer")
 			." SET correct=(SELECT c FROM pc_temp"
 			."     WHERE q=question AND a=answer)"},
-     'postCorrect'=>{'sql'=>"UPDATE ".$self->table("answer")
-		     ." SET correct="
-		     ."(SELECT CASE"
-		     ." WHEN manual >= 0 THEN manual"
-		     ." WHEN total<=0 THEN -1"
-		     ." WHEN black >= ? * total THEN 1"
-		     ." ELSE 0"
-		     ." END AS ticked FROM ".$self->table("zone","capture")
-		     ." WHERE capture_zone.id_a=scoring_answer.question AND capture_zone.id_b=scoring_answer.answer"
-		     ." AND capture_zone.student=? AND capture_zone.copy=? AND capture_zone.type=?)"},
-
      'NEWScore'=>{'sql'=>"INSERT INTO ".$self->table("score")
 		  ." (student,copy,question,score,max,why)"
 		  ." VALUES (?,?,?,?,?,?)"},
@@ -496,7 +497,7 @@ sub define_statements {
       .",(SELECT CASE"
       ."         WHEN manual >= 0 THEN manual"
       ."         WHEN total<=0 THEN -1"
-      ."         WHEN black >= ? * total THEN 1"
+      ."         WHEN black >= ? * total AND black <= ? * total THEN 1"
       ."         ELSE 0"
       ."  END FROM $t_zone"
       ."  WHERE $t_zone.student=? AND $t_zone.copy=? AND $t_zone.type=?"
@@ -519,9 +520,31 @@ sub define_statements {
 		    ." WHERE student=? AND copy=?"},
      'deleteCodes'=>{'sql'=>"DELETE FROM ".$self->table('code')
 		    ." WHERE student=? AND copy=?"},
+     'pagesWhy'=>{'sql'=>"SELECT s.student,s.copy,GROUP_CONCAT(s.why) as why,b.page FROM "
+                  .$self->table('score')." s"
+                  ." JOIN "
+                  ." ( SELECT student,page,question FROM ".$self->table("box","layout")
+                  ."   WHERE role=?"
+                  ."   GROUP BY student,page,question )"
+                  . " b"
+                  ." ON s.student=b.student AND s.question=b.question"
+                  ." GROUP BY s.student,b.page,s.copy"},
     };
 }
 
+# page_why() returns a list of items like
+# {student=>1,copy=>0,page=>1,why=>',V,E,,'}
+# that collects all 'why' attributes for questions that are on each page.
+
+sub pages_why {
+  my ($self)=@_;
+  return(@{$self->dbh->selectall_arrayref($self->statement('pagesWhy'),
+                                          {Slice=>{}},
+                                          BOX_ROLE_ANSWER
+                                         )});
+}
+
+
 # default_strategy($type) returns the default scoring strategy string
 # to be used for questions with type $type (QUESTION_SIMPLE or
 # QUESTION_MULT).
@@ -585,7 +608,8 @@ sub main_strategy_all {
 }
 
 # new_question($student,$question,$type,$indicative,$strategy) adds a
-# question in the database, giving its characteristics
+# question in the database, giving its characteristics. If the
+# question already exists, it is updated with no error.
 
 sub new_question {
   my ($self,$student,$question,$type,$indicative,$strategy)=@_;
@@ -604,7 +628,8 @@ sub question_strategy {
 }
 
 # new_answer($student,$question,$answer,$correct,$strategy) adds an
-# answer in the database, giving its characteristics
+# answer in the database, giving its characteristics. If the answer
+# already exists, it is updated with no error.
 
 sub new_answer {
   my ($self,$student,$question,$answer,$correct,$strategy)=@_;
@@ -798,42 +823,33 @@ sub unalias {
   return($student);
 }
 
-# postcorrect($student,$copy,$darkness_threshold) uses the ticked
-# values from the copy ($student,$copy) (filled by a teacher) to
-# determine which answers are correct for all sheets. This can be used
-# only when the questions/answers are not different from a sheet to
-# another (contrary to the use of random numerical values for
+# postcorrect($student,$copy,$darkness_threshold,$darkness_threshold_up,$set_multiple)
+# uses the ticked values from the copy ($student,$copy) (filled by a
+# teacher) to determine which answers are correct for all sheets. This
+# can be used only when the questions/answers are not different from a
+# sheet to another (contrary to the use of random numerical values for
 # exemple).
 #
-# Here are two SQL implementations for postcorrect: postcorrect_a acts
-# with a single SQL request, whereas postcorrect_b builds a temporary
-# table and requires several SQL requests. postcorrect_b is usualy the
-# fastest, so that postcorrect is only a relay to postcorrect_b. Note
-# that an UPDATE request on a JOINture should be a good idea, but this
-# is not allowed by SQLite.
+# If $set_multiple is true, postcorrect also sets the type of all
+# questions for which 2 or more answers are ticked on the
+# ($student,$copy) answer sheet to be QUESTION_MULT, ans the type of
+# all other questions to QUESTION_SIMPLE.
 
-sub postcorrect_a {
-  my ($self,$student,$copy,$darkness_threshold)=@_;
-  $self->{'data'}->require_module('capture');
-  $self->statement('postCorrect')
-    ->execute($darkness_threshold,$student,$copy,ZONE_BOX);
-}
-
-sub postcorrect_b {
-  my ($self,$student,$copy,$darkness_threshold)=@_;
+sub postcorrect {
+  my ($self,$student,$copy,
+      $darkness_threshold,$darkness_threshold_up,$set_multiple)=@_;
+  die "Missing parameters in postcorrect call"
+    if(!defined($darkness_threshold_up));
   $self->{'data'}->require_module('capture');
   $self->statement('postCorrectNew')->execute();
   $self->statement('postCorrectClr')->execute();
   $self->statement('postCorrectPop')
-    ->execute($darkness_threshold,$student,$copy,ZONE_BOX);
+    ->execute($darkness_threshold,$darkness_threshold_up,$student,$copy,ZONE_BOX);
+  $self->statement('postCorrectMul')->execute(QUESTION_MULT,QUESTION_SIMPLE)
+    if($set_multiple);
   $self->statement('postCorrectSet')->execute();
 }
 
-sub postcorrect {
-  my ($self,@args)=@_;
-  $self->postcorrect_b(@args);
-}
-
 # new_score($student,$copy,$question,$score,$score_max,$why) adds a
 # question score row.
 
@@ -975,9 +991,9 @@ sub student_global {
   return($x=$sth->fetchrow_hashref);
 }
 
-# student_scoring_base($student,$copy,$darkness_threshold) returns
-# useful data to compute questions scores for a particular student
-# (identified by $student and $copy), as a reference to a hash
+# student_scoring_base($student,$copy,$darkness_threshold,$darkness_threshold_up)
+# returns useful data to compute questions scores for a particular
+# student (identified by $student and $copy), as a reference to a hash
 # grouping questions and answers. For exemple :
 #
 # 'main_strategy'=>"",
@@ -996,7 +1012,9 @@ sub student_global {
 # }
 
 sub student_scoring_base {
-  my ($self,$student,$copy,$darkness_threshold)=@_;
+  my ($self,$student,$copy,$darkness_threshold,$darkness_threshold_up)=@_;
+  die "Missing parameters in student_scoring_base call"
+    if(!defined($darkness_threshold_up));
   $self->{'data'}->require_module('capture');
   my $student_strategy=$self->unalias($student);
   my $r={'student_alias'=>$student_strategy,
@@ -1012,7 +1030,8 @@ sub student_scoring_base {
       $r->{'questions'}->{$qa->{'question'}}=$qa;
     }
     $sth=$self->statement('studentAnswersBase');
-    $sth->execute($darkness_threshold,$student,$copy,ZONE_BOX,$s);
+    $sth->execute($darkness_threshold,$darkness_threshold_up,
+		  $student,$copy,ZONE_BOX,$s);
     while(my $qa=$sth->fetchrow_hashref) {
       push @{$r->{'questions'}->{$qa->{'question'}}->{'answers'}},$qa;
     }
