@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2014 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -55,7 +55,7 @@ sub needs_module {
 }
 
 sub build_config_gui {
-  my ($self,$w,$cb)=@_;
+  my ($self,$w,$prefs)=@_;
 }
 
 sub hide {
