@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -20,11 +20,18 @@
 package AMC::Filter;
 
 sub new {
-    my $class = shift;
+    my ($class,%o) = @_;
     my $self={'errors'=>[],
 	      'project_options'=>{},
 	      'filter_results'=>{},
+	      'jobname'=>'',
+	      'jobspecific'=>'',
 	     };
+
+    for my $k (keys %o) {
+	$self->{$k}=$o{$k} if(defined($self->{$k}));
+    }
+
     bless ($self, $class);
     return $self;
 }
@@ -44,6 +51,10 @@ sub errors {
   return(@{$self->{'errors'}});
 }
 
+sub pre_filter {
+  my ($self,$input_file)=@_;
+}
+
 sub filter {
   my ($self,$input_file,$output_file)=@_;
 }
@@ -63,9 +74,14 @@ sub set_filter_result {
   $self->{'filter_results'}->{$name}=$value;
 }
 
+sub get_filter_result {
+  my ($self,$name)=@_;
+  return($self->{'filter_results'}->{$name});
+}
+
 sub unchanged {
   my ($self)=@_;
-  return($self->{'filter_results'}->{'unchanged'});
+  return($self->get_filter_result('unchanged'));
 }
 
 1;
