@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -22,6 +22,7 @@ package AMC::Filter::plain;
 use AMC::Filter;
 use AMC::Basic;
 
+use File::Spec;
 use Data::Dumper;
 
 use utf8;
@@ -40,7 +41,7 @@ sub new {
 				 BoxColor PaperSize
 				 AnswerSheetTitle AnswerSheetPresentation
 				 AnswerSheetColumns
-				 CompleteMulti SeparateAnswerSheet
+				 CompleteMulti SeparateAnswerSheet AutoMarks
 				 DefaultScoringM DefaultScoringS
 				 L-Question L-None L-Name L-Student
 				 LaTeX LaTeX-Preambule LaTeX-BeginDocument
@@ -50,13 +51,17 @@ sub new {
 				 Disable
 				 ManualDuplex SingleSided
 				 L-OpenText L-OpenReserved
+				 CodeDigitsDirection
+				 PackageOptions
+				 NameFieldWidth NameFieldLines NameFieldLinespace
+				 TitleWidth
 				/];
 
     # from these options, which ones are boolean valued?
 
     $self->{'options_boolean'}=[qw/LaTeX xltxtra
 				   ShuffleQuestions QuestionBlocks
-				   CompleteMulti SeparateAnswerSheet
+				   CompleteMulti SeparateAnswerSheet AutoMarks
 				   Arabic
 				   ManualDuplex SingleSided
 				  /];
@@ -77,18 +82,23 @@ sub new {
        'font'=>'Linux Libertine O',
        'arabicfont'=>'Rasheeq',
        'defaultscoringm'=>'haut=2',
-       'l-name'=>__("Name and surname"),
+       'l-name'=>'',
        'l-student'=>__("Please code your student number opposite, and write your name in the box below."),
        'disable'=>'',
        'manualduplex'=>'',
        'singlesided'=>'',
+       'codedigitsdirection'=>'',
+       'namefieldwidth'=>'',
+       'namefieldlines'=>'',
+       'namefieldlinespace'=>'.5em',
+       'titlewidth'=>".47\\linewidth",
       };
 
     # List of modules to be used when parsing (see parse_*
     # corresponding methods for implementation). Modules in the
     # Disable global option won't be used.
 
-    $self->{'parse_modules'}=['local_latex','images','embf','text'];
+    $self->{'parse_modules'}=['local_latex','images','embf','title','text'];
 
     # current question number among questions for which no ID is given
     # in the source
@@ -120,6 +130,10 @@ sub parse_bool {
   }
 }
 
+sub first_defined {
+  for(@_) { return($_) if(defined($_)); }
+}
+
 # transforms the list of current options values to make it coherent
 sub parse_options {
   my ($self)=@_;
@@ -209,98 +223,195 @@ sub check_answers {
   }
 }
 
+sub group_by_id {
+  my ($self,$id,%oo)=@_;
+  my $group='';
+
+  if($id) {
+
+  GROUPS: for(@{$self->{groups}}) {
+      if($_->{id} eq $id) { $group=$_ ; last GROUPS; }
+    }
+
+  } else {
+    $id='unnamed'.$self->unused_letter();
+  }
+
+  if(!$group) {
+    $group=$self->add_group('id'=>$id,'questions'=>[],%oo);
+
+    if($id ne '_main_') {
+      # calls newly created group from the 'main' one
+
+      my $main=$self->group_by_id('_main_');
+
+      add_object($main->{'questions'},
+		 textonly=>1,
+		 text=>{type=>'latex',
+			string=>$self->group_insert_command_name($group)},
+		 %oo,
+		);
+    }
+  }
+
+  return($group);
+}
+
+sub read_options {
+  my ($self,$options_string)=@_;
+  my %oo=();
+  my @opts=split(/,+/,$options_string);
+  for (@opts) {
+    if(/^([^=]+)=(.*)/) {
+      $oo{$1}=$2;
+    } else {
+      $oo{$_}=1;
+    }
+  }
+  return(%oo);
+}
+
 # parse the whole source file to a data tree ($self->{'groups'})
 sub read_source {
   my ($self,$input_file)=@_;
 
-  my %opts=();
-  my $follow=''; # variable where to add content comming from
+  $self->{reader_state}=
+    {
+     follow=>'', # variable where to add content comming from
                  # following lines
-  my $group='';  # current group
-  my $question=''; # current question
+     group=>$self->group_by_id('_main_'),  # current group
+     question=>'', # current question
+    };
+
+  $self->read_file($input_file);
+  $self->check_answers($self->{reader_state}->{question});
+}
+
+sub read_file {
+  my ($self,$input_file)=@_;
 
   # regexp that matches an option name:
   my $opt_re='('.join('|',@{$self->{'options_names'}}).')';
 
-  open(IN,"<:utf8",$input_file);
- LINE: while(<IN>) {
+  debug "Parsing $input_file";
+
+  open(my $infile,"<",$input_file);
+  binmode($infile);
+
+ LINE: while(<$infile>) {
+    if(!utf8::decode($_)) {
+      $self->parse_error(__"Invalid encoding: you must use UTF-8, but your source file was saved using another encoding");
+      last LINE;
+    }
+
     chomp;
 
-    # removes comments
-    s/^\s*\#.*//;
+    debug ":> $_";
 
-    # groups
-    if(/^\s*Group:\s*(.*)/) {
-      if($group && !@{$group->{'questions'}}) {
-# TRANSLATORS: Error text for AMC-TXT parsing, when opening a new group whereas the previous group is empty
- 	$self->parse_error(__"Previous group was empty");
-      }
-      $group=$self->add_group('title'=>$1,'questions'=>[]);
-      $self->value_cleanup($follow);
-      $follow=\$group->{'title'};
-      $self->check_answers($question);
-      $question='';
+    # removes comments
+    if(/^\s*\#/) {
+      debug "Comment";
       next LINE;
     }
 
+    # Insert other file...
+    if(/^\s*IncludeFile:\s*(.*)/i) {
+      my $filename=$1;
+      $filename =~ s/\s+$//;
+      my ($volume,$directories,$file) =
+	File::Spec->splitpath( $input_file );
+      my $dir=File::Spec->catpath($volume,$directories,'');
+      my $f=File::Spec->rel2abs($filename,$dir);
+      debug "Include $f";
+      if(-f $f) {
+	$self->read_file($f);
+      } else {
+	$self->parse_error(sprintf(__("File not found: %s"),$f));
+      }
+    }
+
     # options
     if(/^\s*$opt_re:\s*(.*)/i) {
-      $self->{'options'}->{lc($1)}=$2;
-      $self->value_cleanup($follow);
-      $follow=\$self->{'options'}->{lc($1)};
-      $self->check_answers($question);
-      $question='';
+      debug "Option line ($1)";
+      $self->{options}->{lc($1)}=$2;
+      $self->value_cleanup($self->{reader_state}->{follow});
+      $self->{reader_state}->{follow}=\$self->{'options'}->{lc($1)};
+      $self->check_answers($self->{reader_state}->{question});
+      $self->{reader_state}->{question}='';
       next LINE;
     }
 
-    if(/\s*([a-z0-9-]+):/i) {
+    if(/^([a-z0-9-]+):/i) {
+      debug "Unknown option";
 # TRANSLATORS: Error text for AMC-TXT parsing, when an unknown option is given a value
       $self->parse_error(sprintf(__("Unknown option: %s"),$1));
     }
 
+    # groups
+    if(/^\s*\*([\(\)])(?:\[([^]]*)\])?\s*(.*)/) {
+      my $action=$1;
+      my $options=$2;
+      my $text=$3;
+      debug "Group A=$action O=$options";
+      my %oo=$self->read_options($options);
+      if($action eq '(') {
+	$self->{reader_state}->{group}=
+	  $self->group_by_id($oo{group},
+			     parent=>$self->{reader_state}->{group},
+			     header=>$text,
+			     %oo);
+	$self->{reader_state}->{follow}=\$self->{reader_state}->{group}->{header};
+      } else {
+	$self->{reader_state}->{group}->{footer}=$text;
+	$self->{reader_state}->{follow}=\$self->{reader_state}->{group}->{footer};
+	$self->{reader_state}->{group}=$self->{reader_state}->{group}->{parent};
+      }
+      next LINE;
+    }
+
     # questions
     if(/^\s*(\*{1,2})(?:<([^>]*)>)?(?:\[([^]]*)\])?(?:\{([^\}]*)\})?\s*(.*)/) {
-      $self->check_answers($question);
+      $self->check_answers($self->{reader_state}->{question});
       my $star=$1;
       my $angles=$2;
-      my @opts=split(/,+/,$3);
+      my $options=$3;
       my $scoring=$4;
       my $text=$5;
-      my %oo=();
-      for (@opts) {
-	if(/^([^=]+)=(.*)/) {
-	  $oo{$1}=$2;
-	} else {
-	  $oo{$_}=1;
-	}
+      debug "Question S=$star A=$angles O=$options S=$scoring";
+      my %oo=$self->read_options($options);
+      my $q_group=$self->{reader_state}->{group};
+
+      if($oo{group}) {
+	$q_group=$self->group_by_id($oo{group});
       }
-      if(!$group) {
-	$group=$self->add_group('title'=>'','questions'=>[]);
-      }
-      $question=add_object($group->{'questions'},
-			   'multiple'=>length($star)==2,
-			   'open'=>$angles,
-			   'scoring'=>$scoring,
-			   'text'=>$text,'answers'=>[],%oo);
-      $self->value_cleanup($follow);
-      $follow=\$question->{'text'};
+
+      $self->{reader_state}->{question}=
+	add_object($q_group->{'questions'},
+		   'multiple'=>length($star)==2,
+		   'open'=>$angles,
+		   'scoring'=>$scoring,
+		   'text'=>$text,'answers'=>[],%oo);
+      $self->value_cleanup($self->{reader_state}->{follow});
+      $self->{reader_state}->{follow}=\$self->{reader_state}->{question}->{'text'};
       next LINE;
     }
 
     # answers
     if(/^\s*(\+|-)(?:\[([^]]*)\])?(?:\{([^\}]*)\})?\s*(.*)/) {
-      if($question) {
+      if($self->{reader_state}->{question}) {
 	my $sign=$1;
 	my $letter=$2;
 	my $scoring=$3;
 	my $text=$4;
-	my $a=add_object($question->{'answers'},
+	debug "Choice G=$sign L=$letter S=$scoring";
+	my $a=add_object($self->{reader_state}->{question}->{answers},
 			 'text'=>$text,'correct'=>($sign eq '+'),
 			 'letter'=>$letter,
 			 'scoring'=>$scoring);
-	$self->value_cleanup($follow);
+	$self->value_cleanup($self->{reader_state}->{follow});
 	$follow=\$a->{'text'};
       } else {
+	debug "Choice outside question";
 # TRANSLATORS: Error text for AMC-TXT parsing when a choice is given but no question were opened
 	$self->parse_error(__"Choice outside question");
       }
@@ -308,13 +419,14 @@ sub read_source {
     }
 
     # text following last line
-    if($follow) {
-      $$follow.="\n".$_;
+    if($self->{reader_state}->{follow}) {
+      debug "Follow...";
+      ${$self->{reader_state}->{follow}}.="\n".$_;
     }
   }
-  $self->value_cleanup($follow);
-  $self->check_answers($question);
-  close(IN);
+  debug "Cleanup...";
+  $self->value_cleanup($self->{reader_state}->{follow});
+  close($infile);
 }
 
 # bold font LaTeX command, not used in arabic language because there
@@ -397,29 +509,28 @@ sub parse_local_latex {
   return(@o);
 }
 
-# parse_embf inserts LaTeX commands to switch to italic or bold font
-# when '[_ ... _]' or '[* ... *]' constructs are used in AMC-TXT.
-sub parse_embf {
-  my ($self,@components)=@_;
+# generic code to parse '[xxx ... xxx]' constructs
+sub parse_brackets {
+  my ($self,$modifier,$tex_open,$tex_close,@components)=@_;
   my @o=();
+  my $levels=0;
+  my $tex;
   for my $c (@components) {
     if($c->{'type'} eq 'txt') {
       my $s=$c->{'string'};
-      while($s =~ /\[([_\*])(((?!\]\]).)+?)\1\]/p) {
-	my $modif=$1;
-	my $modif_latex;
-	my $text=$2;
+      while($s =~ /(\[\Q$modifier\E|\Q$modifier\E\])/p) {
+	my $sep=$1;
 	my $before=${^PREMATCH};
 	my $after=${^POSTMATCH};
-	if($modif eq '_') {
-	  $modif_latex='it';
+	if($sep=~/^\[/) {
+	  $tex=$tex_open;
+	  $levels++;
 	} else {
-	  $modif_latex='bf';
+	  $tex=$tex_close;
+	  $levels--;
 	}
 	push @o,{'type'=>'txt','string'=>$before};
-	push @o,{'type'=>'latex','string'=>"\\text".$modif_latex."{"};
-	push @o,$self->parse_embf({'type'=>'txt','string'=>$text});
-	push @o,{'type'=>'latex','string'=>"}"};
+	push @o,{'type'=>'latex','string'=>$tex};
 	$s=$after;
       }
       push @o,{'type'=>'txt','string'=>$s};
@@ -430,6 +541,22 @@ sub parse_embf {
   return(@o);
 }
 
+# parse_embf inserts LaTeX commands to switch to italic or bold font
+# when '[_ ... _]' or '[* ... *]' constructs are used in AMC-TXT.
+sub parse_embf {
+  my ($self,@components)=@_;
+  my @c=$self->parse_brackets('_',"\\textit{","}",@components);
+  @c=$self->parse_brackets('*',"\\textbf{","}",@c);
+  return(@c);
+}
+
+# parse_title inserts LaTeX commands to build a title line from '[==
+# ... ==]' constructs.
+sub parse_title {
+  my ($self,@components)=@_;
+  return($self->parse_brackets('==',"\\AMCmakeTitle{","}",@components));
+}
+
 # parse_text transforms plain text (with no special constructs) to
 # LaTeX code, escaping some characters that have special meaning for
 # LaTeX.
@@ -487,18 +614,33 @@ sub check_latex {
 # the components
 sub format_text {
   my ($self,$t)=@_;
-  $t =~ s/^\s+//;
-  $t =~ s/\s+$//;
+  my $src;
+  if(ref($t) eq 'HASH') {
+    $src=$t;
+  } else {
+    $t =~ s/^\s+//;
+    $t =~ s/\s+$//;
+    $src={'type'=>'txt','string'=>$t};
+  }
 
-  return(join('',$self->check_latex($self->parse_all({'type'=>'txt','string'=>$t}))));
+  return(join('',$self->check_latex($self->parse_all($src))));
 }
 
 # builds the LaTeX scoring command from the question's scoring
 # strategy (or the default scoring strategy)
 sub scoring_string {
   my ($self,$obj,$type)=@_;
-  my $s=$obj->{'scoring'}
-    || $self->{'options'}->{'defaultscoring'.$type};
+  # manual scoring if some scoring was used, either for the question
+  # or for one of the answers
+  my $manual_scoring=$obj->{'scoring'};
+  if($obj->{answers}) {
+    for my $a (@{$obj->{answers}}) {
+      $manual_scoring=1 if($a->{scoring});
+    }
+  }
+  my $s=$obj->{'scoring'};
+  # set to default only if no manual scoring:
+  $s=$self->{'options'}->{'defaultscoring'.$type} if(!$manual_scoring);
   return($s ? "\\scoring{$s}" : "");
 }
 
@@ -515,55 +657,88 @@ sub format_answer {
   return($t);
 }
 
+sub arabic_env {
+  my ($self,$action)=@_;
+  if($self->{'options'}->{'arabic'} && $self->bidi_year()<2011) {
+    return("\\".$action."{arab}");
+  } else {
+    return("");
+  }
+}
+
 # builds the LaTeX code for the question (including answers)
 sub format_question {
   my ($self,$q)=@_;
-  my $qid=$q->{'id'};
-  $qid=$q->{'name'} if(!$qid);
-  $qid=sprintf("Q%03d",++$self->{'qid'}) if(!$qid);
-  my $mult=($q->{'multiple'} ? 'mult' : '');
-  my $ct=($q->{'horiz'} ? 'horiz' : '');
-
   my $t='';
-  $t.="\\begin{arab}"
-    if($self->{'options'}->{'arabic'} && $self->bidi_year()<2011);
-  $t.='\\begin{question'.$mult.'}{'.$qid."}";
-  $t.=$self->scoring_string($q,($q->{'multiple'} ? 'm' : 's'));
-  $t.="\n";
-  $t.=$self->format_text($q->{'text'})."\n";
-  $t.="\\QuestionIndicative\n" if($q->{'indicative'});
-  if($q->{open} ne '') {
-    $t.="\\AMCOpen{".$q->{open}."}{";
+
+  if($q->{textonly}) {
+
+    $t.=$self->arabic_env('begin');
+    $t.=$self->format_text($q->{'text'})."\n";
+    $t.=$self->arabic_env('end');
+
   } else {
-    $t.="\\begin{multicols}{".$q->{'columns'}."}\n"
-      if($q->{'columns'}>1);
-    $t.="\\begin{choices$ct}".($q->{'ordered'} ? "[o]" : "")."\n";
+
+    my $qid=$q->{'id'};
+    $qid=$q->{'name'} if(!$qid);
+    $qid=sprintf("Q%03d",++$self->{'qid'}) if(!$qid);
+    my $mult=($q->{'multiple'} ? 'mult' : '');
+    my $ct=($q->{'horiz'} ? 'horiz' : '');
+
+    $t.=$self->arabic_env('begin');
+    $t.='\\begin{question'.$mult.'}{'.$qid."}";
+    $t.=$self->scoring_string($q,($q->{'multiple'} ? 'm' : 's'));
+    $t.="\n";
+    $t.=$self->format_text($q->{'text'})."\n";
+    $t.="\\QuestionIndicative\n" if($q->{'indicative'});
+    if ($q->{open} ne '') {
+      $t.="\\AMCOpen{".$q->{open}."}{";
+    } else {
+      $t.="\\begin{multicols}{".$q->{'columns'}."}\n"
+	if ($q->{'columns'}>1);
+      $t.="\\begin{choices$ct}".($q->{'ordered'} ? "[o]" : "")."\n";
+    }
+    for my $a (@{$q->{'answers'}}) {
+      $t.=$self->format_answer($a);
+    }
+    if ($q->{open} ne '') {
+      $t.="}\n";
+    } else {
+      $t.="\\end{choices$ct}\n";
+      $t.="\\end{multicols}\n"
+	if ($q->{'columns'}>1);
+    }
+    $t.="\\end{question".$mult."}";
+    $t.=$self->arabic_env('end');
+    $t.="\n";
   }
-  for my $a (@{$q->{'answers'}}) {
-    $t.=$self->format_answer($a);
-  }
-  if($q->{open} ne '') {
-    $t.="}\n";
-  } else {
-    $t.="\\end{choices$ct}\n";
-    $t.="\\end{multicols}\n"
-      if($q->{'columns'}>1);
-  }
-  $t.="\\end{question".$mult."}";
-  $t.="\\end{arab}"
-    if($self->{'options'}->{'arabic'} && $self->bidi_year()<2011);
-  $t.="\n";
+
   return($t);
 }
 
-# returns the group name, defaulting to 'groupX' (where X is a letter
-# beginnig at A)
+sub unused_letter {
+  my ($self)=@_;
+  return(chr(ord("A")+($self->{'letter_i'}++)));
+}
+
+# returns a new group name 'groupX' (where X is a letter beginnig at
+# A)
+sub create_group_name {
+  my ($self)=@_;
+  return( "gr".$self->unused_letter() );
+}
+
+# returns the group name (creating one if necessary)
 sub group_name {
   my ($self,$group)=@_;
-  if(!$group->{'name'}) {
-    $group->{'name'}="group".chr(ord("A")+($self->{'group_number'}++));
+  if(!$group->{name}) {
+    if($group->{id} =~ /^[a-z]+$/i) {
+      $group->{name}=$group->{id};
+    } else {
+      $group->{name}=$self->create_group_name();
+    }
   }
-  return($group->{'name'});
+  return($group->{name});
 }
 
 # gets the Year for the version of installed bidi.sty. This will be
@@ -592,8 +767,18 @@ sub file_header {
   my $t='';
 
   my @package_options=();
+
+  my $o=$self->{'options'}->{packageoptions};
+  $o =~ s/^\s+//;
+  $o =~ s/\s+$//;
+  if($o) {
+    for my $oo (split(/,+/,$o)) {
+      push @package_options,$oo;
+    }
+  }
+
   push @package_options,"bloc" if($self->{'options'}->{'questionblocks'});
-  for my $on (qw/completemulti separateanswersheet/) {
+  for my $on (qw/completemulti separateanswersheet automarks/) {
     push @package_options,$on if($self->{'options'}->{$on});
   }
 
@@ -627,6 +812,7 @@ sub file_header {
   $t .= "\\usepackage{arabxetex}\n"
     if($self->{'options'}->{'arabic'} && $self->bidi_year()>=2011);
   $t .= "\\begin{document}\n";
+  $t .= "\\def\\AMCmakeTitle#1{\\par\\noindent\\hrule\\vspace{1ex}{\\hspace*{\\fill}\\Large\\bf #1\\hspace*{\\fill}}\\vspace{1ex}\\par\\noindent\\hrule\\par\\vspace{1ex}}\n";
   $t .= "\\AMCrandomseed{1527384}\n";
   if($self->{'options'}->{'boxcolor'}) {
     if($self->{'options'}->{'boxcolor'}
@@ -650,10 +836,10 @@ sub file_header {
   $t .= "\\def\\AMCotextReserved{".$self->format_text($self->{'options'}->{'l-openreserved'})."}\n"
     if($self->{'options'}->{'l-openreserved'});
 
-  $t.="\\def\\AMCbeginQuestion#1#2{\\par\\noindent{"
+  $t.="\\def\\AMCbeginQuestion#1#2{\\noindent{"
     .$self->bf_or("\\Large")." "
       .$self->{'options'}->{'l-question'}." #1} #2\\hspace{1em}}\n"
-	."\\def\\AMCformQuestion#1{\\vspace{\\AMCformVSpace}\\par{"
+	."\\def\\AMCformQuestion#1{{"
 	  .$self->bf_or("\\Large")." "
 	    .$self->{'options'}->{'l-question'}." #1 :}}\n"
 	      if($self->{'options'}->{'l-question'});
@@ -702,14 +888,22 @@ sub page_header {
 
 # Builds and returns the name field LaTeX code
 sub full_namefield {
-  my ($self,$n_ligs)=@_;
+  my ($self,$with_code)=@_;
+  my $n_ligs;
+  if($self->{'options'}->{'namefieldlines'}) {
+    $n_ligs=$self->{'options'}->{'namefieldlines'};
+  } else {
+    $n_ligs=($with_code ? 2 : 1);
+  }
   my $t='';
   $t.="\\namefield{\\fbox{";
   $t.="\\begin{minipage}{.9\\linewidth}"
-    .$self->{'options'}->{'l-name'}
-	.("\n\n\\vspace*{.5cm}\\dotfill" x $n_ligs)
-	  ."\n\\vspace*{1mm}"
-	    ."\n\\end{minipage}";
+    .($self->{'options'}->{'l-name'}
+      ? $self->{'options'}->{'l-name'}
+      : '\\AMClocalized{namesurname}')
+      .(("\n\n\\vspace*{".$self->{'options'}->{'namefieldlinespace'}."}\\dotfill") x $n_ligs)
+	."\n\\vspace*{1mm}"
+	  ."\n\\end{minipage}";
   $t.="\n}}";
   return($t);
 }
@@ -724,26 +918,33 @@ sub student_block {
     # Header layout with a code (student number)
 
     my $vertical=($self->{'options'}->{'code'}>$self->{'maxhorizcode'});
+    $vertical=1 if($self->{options}->{codedigitsdirection} =~ /^v/i);
+    $vertical=0 if($self->{options}->{codedigitsdirection} =~ /^h/i);
 
     $t.="{\\setlength{\\parindent}{0pt}\\hspace*{\\fill}";
-    $t.=($vertical?"":"\\hbox{\\vbox{");
+    $t.=($vertical?"":"\\hbox{\\vtop{");
     $t.= "\\LR{" if($self->{'options'}->{'arabic'} && $vertical);
     $t.="\\AMCcode".($vertical ? "" : "H")."{student.number}{".
 	$self->{'options'}->{'code'}."}";
     $t.= "}" if($self->{'options'}->{'arabic'} && $vertical);
     $t.=($vertical?"":"}}")."\\hspace*{\\fill}"
-      ."\\begin{minipage}".($vertical?"[b]":"")."{5.8cm}"
+      ."\\begin{minipage}".($vertical?"[b]":"[t]")."{"
+      .($self->{'options'}->{'namefieldwidth'} ?
+	$self->{'options'}->{'namefieldwidth'} : '5.8cm')
+	."}"
 	."\$\\long".($self->{'options'}->{'arabic'} ? "right" : "left")
 	  ."arrow{}\$\\hspace{0pt plus 1cm}"
 	    .$self->{'options'}->{'l-student'}
 	      ."\\vspace{3ex}\n\n\\hfill"
-		.$self->full_namefield(2)
+		.$self->full_namefield(1)
 		  ."\\hfill\\vspace{5ex}\\end{minipage}\\hspace*{\\fill}"
 		    ."\n\n}";
     $t.="\\vspace{4mm}\n";
   } else {
     # header layout without code
-    $t.= "\\begin{minipage}{.47\\linewidth}\n";
+    $t.= "\\begin{minipage}{"
+      .$self->{'options'}->{'titlewidth'}
+      ."}\n";
     my $titlekey=($self->{'options'}->{'separateanswersheet'}
 		  ? 'answersheettitle' : 'title');
     if($self->{'options'}->{$titlekey}) {
@@ -752,13 +953,50 @@ sub student_block {
 	  ."\\end{center}\n\n";
     }
     $t.= "\\end{minipage}\\hfill\n";
-    $t.= "\\begin{minipage}{.47\\linewidth}\n";
-    $t.= $self->full_namefield(1);
+    $t.= "\\begin{minipage}{".
+      ($self->{'options'}->{'namefieldwidth'} ?
+	$self->{'options'}->{'namefieldwidth'} : ".47\\linewidth")
+      ."}\n";
+    $t.= $self->full_namefield(0);
     $t.= "\\end{minipage}\\vspace{4mm}\n\n";
   }
   return($t);
 }
 
+sub group_insert_command_name {
+  my ($self,$group)=@_;
+  return("\\insertPlainGroup".$self->group_name($group));
+}
+
+sub group_insert_command_def {
+  my ($self,$group)=@_;
+  my $t="\\def".$self->group_insert_command_name($group)."{";
+  if($group->{header}) {
+    $t.="\\noindent ".$self->format_text($group->{header});
+    $t.="\\vspace{1.5ex}\\par\n" if(!$group->{custom}
+			   && $group->{columns}<=1);
+  }
+  $t.="\\begin{multicols}{".$group->{columns}."}"
+    if($group->{columns}>1);
+  for my $q (grep { $_->{first} } (@{$group->{questions}})) {
+    $t.=$self->format_question($q)."\n";
+  }
+  $t.="\\insertgroup";
+  $t.="[".$group->{numquestions}."]" if($group->{numquestions});
+  $t.="{".$self->group_name($group)."}";
+  for my $q (grep { $_->{last} } (@{$group->{questions}})) {
+    $t.="\n".$self->format_question($q);
+  }
+  $t.="\\end{multicols}"
+    if($group->{columns}>1);
+  if($group->{footer}) {
+    $t.=$self->format_text($group->{footer});
+    $t.="\\vspace{1.5ex}\\par\n" if(!$group->{custom} && !$group->{cut});
+  }
+  $t.="}\n";
+  return($t);
+}
+
 # writes the LaTeX output file
 sub write_latex {
   my ($self,$output_file)=@_;
@@ -767,14 +1005,26 @@ sub write_latex {
 
   print OUT $self->file_header();
 
-  if($self->{'options'}->{'shufflequestions'}) {
-    for my $group (@{$self->{'groups'}}) {
-      for my $question (@{$group->{'questions'}}) {
-	print OUT "\\element{".$self->group_name($group)."}{\n";
-	print OUT $self->format_question($question);
-	print OUT "}\n";
+  for my $group (@{$self->{'groups'}}) {
+
+    # create group elements from the questions
+
+    my @questions=grep { !($_->{first} || $_->{last}) }
+      @{$group->{'questions'}};
+    my $q;
+    while($q=shift @questions) {
+      print OUT "\\element{".$self->group_name($group)."}{\n";
+      print OUT $self->format_question($q);
+      while(@questions && $questions[0]->{'next'}) {
+	print OUT "\n";
+	print OUT $self->format_question(shift @questions);
       }
+      print OUT "}\n";
     }
+
+    # command to print the group
+
+    print OUT $self->group_insert_command_def($group);
   }
 
   print OUT "\\onecopy{5}{\n";
@@ -791,36 +1041,30 @@ sub write_latex {
   print OUT "\\end{arab}" if($self->{'options'}->{'arabic'});
   print OUT "\n\n";
 
-  for my $group (@{$self->{'groups'}}) {
-    if($group->{'title'}) {
-      print OUT "\\begin{center}\\hrule\\vspace{2mm}"
-	.$self->bf_or("\\Large","\\bf\\Large")." ".
-	  $self->format_text($group->{'title'})."\\vspace{1mm}\\hrule\\end{center}\n\n";
-    }
-    print OUT "\\begin{arab}"
-      if($self->{'options'}->{'arabic'} && $self->bidi_year()>=2011);
-    if($self->{'options'}->{'columns'}>1) {
-      print OUT "\\begin{multicols}{".$self->{'options'}->{'columns'}."}\n";
-    } else {
-      print OUT "\\vspace{2ex}\n\n";
-    }
-
-    if($self->{'options'}->{'shufflequestions'}) {
-      print OUT "\\shufflegroup{".$self->group_name($group)."}\n";
-      print OUT "\\insertgroup{".$self->group_name($group)."}\n";
-    } else {
-      for my $question (@{$group->{'questions'}}) {
-	print OUT $self->format_question($question);
-      }
-    }
-
-    if($self->{'options'}->{'columns'}>1) {
-      print OUT "\\end{multicols}\n";
-    }
-    print OUT "\\end{arab}"
-      if($self->{'options'}->{'arabic'} && $self->bidi_year()>=2011);
+  # shuffle groups...
+  for my $g (@{$self->{groups}}) {
+    print OUT "\\shufflegroup{".$self->group_name($g)."}\n"
+      if(parse_bool(first_defined($g->{shuffle},$self->{'options'}->{'shufflequestions'})));
   }
 
+  # print groups
+  print OUT "\\begin{arab}"
+    if($self->{'options'}->{'arabic'} && $self->bidi_year()>=2011);
+  if($self->{'options'}->{'columns'}>1) {
+    print OUT "\\begin{multicols}{".$self->{'options'}->{'columns'}."}\n";
+  } else {
+    print OUT "\\vspace{2ex}\n\n";
+  }
+
+  print OUT $self->group_insert_command_name($self->group_by_id('_main_'))."\n";
+
+  if($self->{'options'}->{'columns'}>1) {
+    print OUT "\\end{multicols}\n";
+  }
+  print OUT "\\end{arab}"
+    if($self->{'options'}->{'arabic'} && $self->bidi_year()>=2011);
+
+  # separate answer sheet
   if($self->{'options'}->{'separateanswersheet'}) {
     if($self->{'options'}->{'singlesided'}) {
       print OUT "\n\\clearpage\n\n";
