@@ -1,6 +1,6 @@
 #! /usr/bin/perl -w
 #
-# Copyright (C) 2008-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -25,6 +25,7 @@ use AMC::Gui::PageArea;
 use AMC::Data;
 use AMC::DataModule::capture ':zone';
 use AMC::NamesFile;
+use AMC::Gui::WindowSize;
 
 use Getopt::Long;
 
@@ -39,6 +40,9 @@ use constant {
     COPIES_MANUEL => 4,
     COPIES_BG => 5,
     COPIES_IIMAGE => 6,
+
+    NAMES_NAME => 0,
+    NAMES_I => 1,
 };
 
 use_gettext;
@@ -58,6 +62,7 @@ sub new {
 	      'data'=>'',assoc=>'','capture'=>'','layout'=>'',
 	      'global'=>0,
 	      'show_all'=>1,
+	      'complete_beginning'=>1,
 	      'encodage_liste'=>'UTF-8',
 	      'separateur'=>"",
 	      'identifiant'=>'',
@@ -139,7 +144,7 @@ sub new {
     $self->{'gui'}->set_translation_domain('auto-multiple-choice');
     $self->{'gui'}->add_from_file($glade_xml);
 
-    for my $k (qw/general tableau titre photo associes_cb copies_tree bouton_effacer bouton_inconnu scrolled_tableau viewport_tableau button_show_all/) {
+    for my $k (qw/general tableau titre photo associes_cb copies_tree bouton_effacer bouton_inconnu scrolled_tableau viewport_tableau button_show_all student_typein v_complete_beginning/) {
 	$self->{$k}=$self->{'gui'}->get_object($k);
     }
 
@@ -149,6 +154,12 @@ sub new {
 
     AMC::Gui::PageArea::add_feuille($self->{'photo'});
 
+    $self->{'names_model'}=Gtk2::ListStore->new ('Glib::String',
+						 'Glib::String',
+						 );
+
+    $self->initial_size;
+
     my @bouton_nom=();
     my @bouton_eb=();
     $self->{'boutons'}=\@bouton_nom;
@@ -161,9 +172,13 @@ sub new {
     for my $i (0..($self->{'liste'}->taille()-1)) {
 	my $eb=Gtk2::EventBox->new();
 	my $b=Gtk2::Button->new();
-	my $l=Gtk2::Label->new($self->{'liste'}->data_n($i,'_ID_'));
+	my $name=$self->{'liste'}->data_n($i,'_ID_');
+	my $l=Gtk2::Label->new($name);
+	$self->{'names_model'}->insert_with_values($i,
+						   NAMES_NAME,$name,
+						   NAMES_I,$i);
 	$b->add($l);
-	$b->set_tooltip_text($self->{'liste'}->data_n($i,'_ID_'));
+	$b->set_tooltip_text($name);
 	$l->set_size_request(10,-1);
 	if($self->{'rtl'}
 	   && $self->{'general'}->get_direction() eq 'rtl') {
@@ -173,8 +188,6 @@ sub new {
 
 	push @bouton_nom,$b;
 	push @bouton_eb,$eb;
-	$b->show_all();
-	$eb->show();
 	$b->signal_connect (clicked => sub { $self->choisit($i) });
 	$b->set_focus_on_click(0);
 	$self->style_bouton($i);
@@ -236,19 +249,29 @@ sub new {
     my $ii=0;
     for my $i (@images) {
       my @sc=($i->{'student'},$i->{'copy'});
-      my $iter=$copies_store->append();
-      $copies_store->set($iter,
-			 COPIES_N,studentids_string(@sc),
-			 COPIES_STUDENT,$sc[0],
-			 COPIES_COPY,$sc[1],
-			 COPIES_AUTO,$self->{'assoc'}->get_auto(@sc),
-			 COPIES_MANUEL,$self->{'assoc'}->get_manual(@sc),
-			 COPIES_IIMAGE,$ii,
-			);
+      $copies_store->insert_with_values
+	($ii,
+	 COPIES_N,studentids_string(@sc),
+	 COPIES_STUDENT,$sc[0],
+	 COPIES_COPY,$sc[1],
+	 COPIES_AUTO,$self->{'assoc'}->get_auto(@sc),
+	 COPIES_MANUEL,$self->{'assoc'}->get_manual(@sc),
+	 COPIES_IIMAGE,$ii,
+	);
       $ii++;
     }
     $self->{'assoc'}->end_transaction('ALST');
 
+    # auto-completion
+
+    $self->{'completion'}=Gtk2::EntryCompletion->new();
+    $self->{'completion'}->set_model($self->{'names_model'});
+    $self->{'completion'}->set_text_column(NAMES_NAME);
+    $self->{'completion'}->set_minimum_key_length(2);
+    $self->{'completion'}->set_match_func(\&compare_names,$self);
+    $self->{'completion'}->signal_connect("match-selected",\&select_from_entry,$self);
+    $self->{'student_typein'}->set_completion($self->{'completion'});
+
     # retenir...
 
     $self->{'images'}=\@images;
@@ -264,11 +287,36 @@ sub new {
     $self->maj_couleurs_liste();
     $self->{'assoc'}->end_transaction('ANCL');
 
-    $self->initial_size;
-
     return($self);
 }
 
+# function used to look at a key from the names, for auto-completion
+#
+# {X:X}
+sub compare_names {
+  my ($widget,$key,$iter,$self)=@_;
+  my $name=$self->{'names_model'}->get($iter,NAMES_NAME);
+
+  if($self->{'complete_beginning'}) {
+    return($name =~ /^\Q$key\E/i);
+  } else {
+    return($name =~ /\Q$key\E/i);
+  }
+}
+
+# callback when selecting a completion match from the entry
+#
+# {X:X}
+sub select_from_entry {
+  my ($widget,$model,$iter,$self)=@_;
+  my $i=$model->get($iter,NAMES_I);
+  my $name=$model->get($iter,NAMES_NAME);
+  debug "Selecting from entry auto-completion: I=$i NAME=$name";
+  $self->choisit($i);
+  $self->{'student_typein'}->set_text('');
+  return(1);
+}
+
 # is "show all" button active?
 #
 # {X:X}
@@ -288,10 +336,12 @@ sub set_n_cols {
   $self->{'general'}->window()->set_cursor($self->{'cursor_watch'});
   Gtk2->main_iteration while ( Gtk2->events_pending );
 
-  $self->{'tableau'}->resize($self->{'assoc-ncols'},$nligs);
+  $self->{'tableau'}->hide();
 
   $self->{'tableau'}->foreach(sub { $self->{'tableau'}->remove(shift); });
 
+  $self->{'tableau'}->resize($self->{'assoc-ncols'},$nligs);
+
   my $x=0;
   my $y=0;
   my $i=-1;
@@ -307,6 +357,8 @@ sub set_n_cols {
   }
   $self->{'scrolled_tableau'}->set_policy('never','automatic');
 
+  $self->{'tableau'}->show_all();
+
   Gtk2->main_iteration while ( Gtk2->events_pending );
   $self->{'general'}->window()->set_cursor(undef);
 }
@@ -339,16 +391,24 @@ sub set_show_all {
   $self->set_n_cols();
 }
 
+# Gets state of "Beggining" checkbox
+#
+# {X:X}
+sub set_complete_beginning {
+  my ($self)=@_;
+  $self->{'complete_beginning'}=$self->{'v_complete_beginning'}->get_active();
+}
+
 # Sets the window size to requested one (saved the last time the
 # window was used)
 #
 # {X:X}
 sub initial_size {
   my ($self)=@_;
-  if($self->{'size_prefs'} && 
-     $self->{'size_prefs'}->{'assoc_window_size'} =~ /^([0-9]+)x([0-9]+)$/) {
-    Gtk2->main_iteration while ( Gtk2->events_pending );
-    $self->{'general'}->resize($1,$2);
+  if($self->{'size_prefs'}) {
+    AMC::Gui::WindowSize::size_monitor
+	($self->{'general'},{env=>$self->{'size_prefs'},
+			     key=>'assoc_window_size'});
   }
 }
 
@@ -398,38 +458,18 @@ sub maj_contenu_liste_sc {
 #
 # {IN:}
 sub maj_couleurs_liste { # mise a jour des couleurs la liste
-    my ($self)=@_;
-
-    for my $ii (0..$#{$self->{'images'}}) {
-      my @sc=$self->image_sc($ii);
-      my $iter=model_id_to_iter($self->{'copies_store'},COPIES_IIMAGE,$ii);
-      if($iter) {
-	my $etat=$self->{'assoc'}->state(@sc);
-	my $coul;
-	if($etat==0) {
-	  my $x=$self->{'assoc'}->get_manual(@sc);
-	  if(defined($x) && $x eq 'NONE') {
-	    $coul='salmon';
-	  } else {
-	    $coul=undef;
-	  }
-	} elsif($etat==1) {
-	  if($self->{'assoc'}->get_manual(@sc)) {
-	    $coul='lightgreen';
-	  } else {
-	    $coul='lightblue';
-	  }
-	} else {
-	  $coul='salmon';
-	}
-	$self->{'copies_store'}->set($iter,
-				     COPIES_BG,$coul,
-				    );
-      } else {
-	debug_and_stderr "*** [color] no iter for image $ii, sheet "
-	  .studentids_string(@sc)." ***\n";
-      }
-    }
+  my ($self)=@_;
+  my $counts=$self->{'assoc'}->counts_hash();
+  my $iter=$self->{'copies_store'}->get_iter_first();
+  while(defined($iter)) {
+    my @sc=$self->{'copies_store'}->get($iter,COPIES_STUDENT,COPIES_COPY);
+    $self->{'copies_store'}
+      ->set($iter,
+            COPIES_BG,
+            $counts->{studentids_string(@sc)}->{color},
+           );
+    $iter=$self->{'copies_store'}->iter_next($iter);
+  }
 }
 
 # Quits.
@@ -439,13 +479,6 @@ sub quitter {
     if($self->{'global'}) {
 	Gtk2->main_quit;
     } else {
-      if($self->{'size_prefs'}) {
-	my $dims=join('x',$self->{'general'}->get_size);
-	if($dims ne $self->{'size_prefs'}->{'assoc_window_size'}) {
-	  $self->{'size_prefs'}->{'assoc_window_size'}=$dims;
-	  $self->{'size_prefs'}->{'_modifie_ok'}=1;
-	}
-      }
       $self->{'general'}->destroy;
       &{$self->{'fin'}}($self);
     }
@@ -476,6 +509,7 @@ sub sc2inom {
     if($code) {
       return($self->{'liste'}->data($self->{'liste_key'},
 				    $code,
+                                    test_numeric=>1,
 				    'all'=>1,'i'=>1));
     } else {
       return();
@@ -684,7 +718,7 @@ sub charge_image {
       $self->vraie_copie(1);
     } else {
 	$i=-1;
-	$self->{'photo'}->set_image();
+	$self->{'photo'}->set_image('');
 	$self->vraie_copie(0);
     }
     $self->{'iimage'}=$i;
@@ -718,7 +752,7 @@ sub image_suivante {
 
     $self->{'assoc'}->begin_read_transaction('ALIS');
     while($i != $self->{'iimage'}
-	  && ($self->{'assoc'}->get_real($self->image_sc($i))
+	  && ($self->{'assoc'}->get_real($self->image_sc($i)) ne ''
 	      && ! $self->{'associes_cb'}->get_active()) ) {
 	$i=$self->i_suivant($i,$pas);
 	if($pas==1) {
@@ -811,7 +845,7 @@ sub style_bouton {
 sub style_bouton_code {
   my ($self,$code,$actif)=@_;
   for($self->{'liste'}->data($self->{'liste_key'},$code,
-			     'all'=>1,'i'=>1)) {
+			     test_numeric=>1,'all'=>1,'i'=>1)) {
     $self->style_bouton($_,$actif);
   }
 }
