@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2008,2012-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2014 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -36,6 +36,9 @@ sub new {
 	$self->{$_}=$o{$_} if(defined($self->{$_}));
     }
 
+    debug "Create progression pipe for <".$self->{id}."> up to $entier".
+      ($self->{bar}?" (progress bar side)":"");
+
     bless $self;
     $|++ if($self->{'id'});
     return($self);
