@@ -1,6 +1,6 @@
 #! /usr/bin/perl -w
 #
-# Copyright (C) 2008-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2014 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -76,6 +76,11 @@ sub erreurs {
     return($self->get_messages('ERR'));
 }
 
+sub warnings {
+    my ($self)=(@_);
+    return($self->get_messages('WARN'));
+}
+
 sub variables {
     my ($self)=(@_);
     return(%{$self->{'variables'}});
@@ -88,10 +93,16 @@ sub variable {
 
 sub quitte {
     my ($self)=(@_);
+
+    $self->{closing}=1;
+    $self->stop_watch();
+
     my $pid=$self->proc_pid();
     debug "Canceling command [".$self->{'signal'}."->".$pid."].";
 
     kill $self->{'signal'},$pid if($pid =~ /^[0-9]+$/);
+
+    $self->close(cancelled=>1);
 }
 
 sub open {
@@ -126,28 +137,44 @@ sub open {
     }
 }
 
+sub stop_watch {
+  my ($self)=@_;
+
+  if($self->{tag}) {
+    Gtk2::Helper->remove_watch( $self->{tag} );
+    $self->{tag}='';
+  }
+}
+
+sub close {
+  my ($self,%data)=@_;
+
+  $self->stop_watch();
+
+  close($self->{'fh'});
+
+  debug "Command [".$self->{'pid'}."] : OK - ".($self->n_messages('ERR'))." erreur(s)\n";
+
+  my @tb=times();
+  debug sprintf("Total parent exec times during ".$self->{pid}.": [%7.02f,%7.02f]",$tb[0]+$tb[1]-$self->{'times'}->[0]-$self->{'times'}->[1],$tb[2]+$tb[3]-$self->{'times'}->[2]-$self->{'times'}->[3]);
+
+  $self->{'pid'}='';
+  $self->{'tag'}='';
+  $self->{'fh'}='';
+
+  $self->{'avancement'}->set_text('');
+
+  &{$self->{'finw'}}($self,%data) if($self->{'finw'});
+  &{$self->{'fin'}}($self,%data) if($self->{'fin'});
+}
 
 sub get_output {
     my ($self)=@_;
 
+    return if($self->{closing});
+
     if( eof($self->{'fh'}) ) {
-        Gtk2::Helper->remove_watch( $self->{'tag'} );
-	  close($self->{'fh'});
-
-	  debug "Command [".$self->{'pid'}."] : OK - ".($self->n_messages('ERR'))." erreur(s)\n";
-
-	  my @tb=times();
-	  debug sprintf("Total parent exec times during ".$self->{pid}.": [%7.02f,%7.02f]",$tb[0]+$tb[1]-$self->{'times'}->[0]-$self->{'times'}->[1],$tb[2]+$tb[3]-$self->{'times'}->[2]-$self->{'times'}->[3]);
-
-	  $self->{'pid'}='';
-	  $self->{'tag'}='';
-	  $self->{'fh'}='';
-
-	  $self->{'avancement'}->set_text('');
-
-	  &{$self->{'finw'}}($self) if($self->{'finw'});
-	  &{$self->{'fin'}}($self) if($self->{'fin'});
-
+      $self->close();
     } else {
 	my $fh=$self->{'fh'};
 	my $line = decode("utf8",<$fh>);
