@@ -1,6 +1,6 @@
 #! /usr/bin/perl -w
 #
-# Copyright (C) 2008-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -27,6 +27,7 @@ use XML::Simple;
 use File::Spec::Functions qw/splitpath catpath splitdir catdir catfile rel2abs tmpdir/;
 use File::Temp qw/ tempfile tempdir /;
 
+use AMC::Path;
 use AMC::Basic;
 use AMC::Gui::PageArea;
 use AMC::Gui::WindowSize;
@@ -44,6 +45,7 @@ use constant {
     MDIAG_STUDENT => 7,
     MDIAG_PAGE => 8,
     MDIAG_COPY => 9,
+    MDIAG_WHY => 10,
 };
 
 use_gettext;
@@ -56,6 +58,7 @@ sub new {
 	      'etud'=>'',
 	      'dpi'=>75,
 	      'seuil'=>0.1,
+	      'seuil_up'=>1.0,
 	      'seuil_sens'=>8.0,
 	      'seuil_eqm'=>3.0,
 	      'fact'=>1/4,
@@ -69,6 +72,8 @@ sub new {
 	      'multiple'=>0,
 	      'onscan'=>'',
 	      'size_monitor'=>'',
+              'invalid_color_name'=>"#FFEF3B",
+              'empty_color_name'=>"#78FFED",
 	  };
 
     for (keys %o) {
@@ -82,6 +87,8 @@ sub new {
     $self->{'data'}=AMC::Data->new($self->{'data-dir'});
     $self->{'layout'}=$self->{'data'}->module('layout');
     $self->{'capture'}=$self->{'data'}->module('capture');
+    $self->{'scoring'}=$self->{'data'}->module('scoring');
+    $self->{'scoring'}=$self->{'data'}->module('scoring');
 
     die "No PDF subject file" if(! $self->{'sujet'});
     die "Subject file ".$self->{'sujet'}." not found" if(! -f $self->{'sujet'});
@@ -101,7 +108,7 @@ sub new {
     $self->{'gui'}->set_translation_domain('auto-multiple-choice');
     $self->{'gui'}->add_from_file($glade_xml);
 
-    for my $k (qw/general area navigation_h navigation_v goto goto_v diag_tree button_photocopy scan_view/) {
+    for my $k (qw/general area navigation_h navigation_v goto goto_v diag_tree button_photocopy scan_view navigate/) {
 	$self->{$k}=$self->{'gui'}->get_object($k);
     }
 
@@ -116,6 +123,15 @@ sub new {
       $self->{'scan_view_model'}=cb_model(0,__("Original"),
 					  1,__("Scan"));
       $self->{'scan_view'}->set_model($self->{'scan_view_model'});
+
+# TRANSLATORS: This is one of the choices for pages navigation in manual data capture window. Here, navigation goes through all pages. Please keep this text very short (say less than 5 letters) so that the window is not too large
+      $self->{navigate_model}=cb_model(0,__("all"),
+ # TRANSLATORS: This is one of the choices for pages navigation in manual data capture window. Here, navigation goes through pages with some invalid answers. Please keep this text very short (say less than 5 letters) so that the window is not too large
+                                       1,__("inv"),
+# TRANSLATORS: This is one of the choices for pages navigation in manual data capture window. Here, navigation goes through pages with empty or invalid answers. Please keep this text very short (say less than 5 letters) so that the window is not too large
+                                       2,__("i&e"));
+      $self->{navigate}->set_model($self->{navigate_model});
+
       $self->{'navigation_h'}->show();
     }
 
@@ -126,6 +142,8 @@ sub new {
     AMC::Gui::PageArea::add_feuille
 	($self->{'area'},'',
 	 'yfactor'=>2,
+         invalid_color_name=>$self->{invalid_color_name},
+         empty_color_name=>$self->{empty_color_name},
 	 'editable'=>$self->{'editable'},
 	 'marks'=>($self->{'editable'} ? '' : 'blue'));
 
@@ -193,6 +211,7 @@ sub new_diagstore {
 				      'Glib::String',
 				      'Glib::String',
 				      'Glib::String',
+				      'Glib::String',
 				     );
   $diag_store->set_sort_func(MDIAG_EQM,\&sort_num,MDIAG_EQM);
   $diag_store->set_sort_func(MDIAG_DELTA,\&sort_num,MDIAG_DELTA);
@@ -296,9 +315,13 @@ sub maj_list_all {
   $self->{'capture'}->begin_read_transaction;
   my $summary=$self->{'capture'}
     ->summaries('darkness_threshold'=>$self->{'seuil'},
+		'darkness_threshold_up'=>$self->{'seuil_up'},
 		'sensitivity_threshold'=>$self->{'seuil_sens'},
-		'mse_threshold'=>$self->{'seuil_eqm'});
-  my $capture_free=$self->{'capture'}->no_capture_pages;
+		'mse_threshold'=>$self->{'seuil_eqm'},
+                'why'=>1);
+  my $capture_free=[];
+  push @$capture_free,@{$self->{'capture'}->question_only_pages} if(!$self->{editable});
+  push @$capture_free,@{$self->{'capture'}->no_capture_pages};
   $self->{'capture'}->end_transaction;
 
   $diag_store = $self->new_diagstore;
@@ -318,8 +341,9 @@ sub maj_list_all {
 	    MDIAG_EQM,$p->{'mse_string'},
 	    MDIAG_EQM_BACK,$p->{'mse_color'},
 	    MDIAG_DELTA,$p->{'sensitivity_string'},
-	    MDIAG_DELTA_BACK,$p->{'sensitivity_color'},
-	   );
+            MDIAG_DELTA_BACK,$p->{'sensitivity_color'},
+            MDIAG_WHY,$p->{why},
+           );
     $select_iid=$i if($select_spc &&
 		      $select_spc->[0]==$p->{'student'} &&
 		      $select_spc->[1]==$p->{'page'} &&
@@ -339,7 +363,8 @@ sub maj_list_all {
 	    MDIAG_EQM,'',
 	    MDIAG_EQM_BACK,undef,
 	    MDIAG_DELTA,'',
-	    MDIAG_DELTA_BACK,undef,
+            MDIAG_DELTA_BACK,undef,
+            MDIAG_WHY,'',
 	   );
     $select_iid=$i if($select_spc &&
 		      $select_spc->[0]==$p->[0] &&
@@ -371,6 +396,7 @@ sub maj_list_i {
       ->page_summary(@spc,
 		     'mse_threshold'=>$self->{'seuil_eqm'},
 		     'darkness_threshold'=>$self->{'seuil'},
+		     'darkness_threshold_up'=>$self->{'seuil_up'},
 		     'sensitivity_threshold'=>$self->{'seuil_sens'},
 		    );
     $self->{'capture'}->end_transaction('lUPD');
@@ -491,7 +517,7 @@ sub charge_i {
 
       debug "Getting layout info";
 
-      for (qw/box namefield digit/) { $self->{'layinfo'}->{$_}=[]; }
+      for (qw/box questionbox namefield digit scorezone/) { $self->{'layinfo'}->{$_}=[]; }
 
       if($self->{'onscan'}) {
 	my %ci=();
@@ -511,7 +537,7 @@ sub charge_i {
 	my $c;
 	my $sth;
 
-	for my $type (qw/box digit namefield/) {
+	for my $type (qw/box questionbox scorezone digit namefield/) {
 	  for my $c ($self->{'layout'}->type_info($type,@ep)) {
 	    push @{$self->{'layinfo'}->{$type}},{%$c};
 	  }
@@ -520,18 +546,26 @@ sub charge_i {
 	$self->{'layinfo'}->{'page'}=$self->{'layout'}->page_info(@ep);
       }
 
-      # mise a jour des cases suivant saisies deja presentes
+      if($self->{editable}) {
+	# gets scoring results to highlight empty and invalid answers
 
-      for my $i (@{$self->{'layinfo'}->{'box'}}) {
-	my $id=$i->{'question'}."."
-	  .$i->{'answer'};
-	my $t=$self->{'capture'}
-	  ->ticked(@spc[0,2],$i->{'question'},$i->{'answer'},
-		   $self->{'seuil'});
-	$t='' if(!defined($t));
-	debug "Q=$id R=$t";
-	$i->{'id'}=[@spc];
-	$i->{'ticked'}=$t;
+	for my $a (@{$self->{layinfo}->{box}}) {
+	  $a->{scoring}=$self->{scoring}->question_result(@spc[0,2],$a->{question});
+	}
+
+	# mise a jour des cases suivant saisies deja presentes
+
+	for my $i (@{$self->{'layinfo'}->{'box'}}) {
+	  my $id=$i->{'question'}."."
+	    .$i->{'answer'};
+	  my $t=$self->{'capture'}
+	    ->ticked(@spc[0,2],$i->{'question'},$i->{'answer'},
+		     $self->{'seuil'},$self->{'seuil_up'});
+	  $t='' if(!defined($t));
+	  debug "Q=$id R=$t";
+	  $i->{'id'}=[@spc];
+	  $i->{'ticked'}=$t;
+	}
       }
     }
 
@@ -585,28 +619,41 @@ sub synchronise {
     $self->{'area'}->sync();
 
     $self->maj_list_i;
+  }
+
+sub navigate_ok {
+  my ($self,$nav_mode,$path)=@_;
+  return(1) if($nav_mode==0);
+  my $iter=$self->{diag_store}->get_iter($path);
+  my $why=$self->{diag_store}->get($iter,MDIAG_WHY);
+  return(0) if($nav_mode==1 && $why !~ /E/);
+  return(0) if($nav_mode==2 && $why !~ /[EV]/);
+  return(1);
 }
 
 sub passe_precedent {
   my ($self)=@_;
+  my $nav=$self->{navigate}->get_active();
   my ($path)=$self->{'diag_tree'}->get_cursor();
   if($path) {
-    if($path->prev) {
-      $self->{'diag_tree'}->set_cursor($path);
-    }
+    my $ok;
+    do {
+      $ok=$path->prev();
+    } while($ok && !$self->navigate_ok($nav,$path));
+    $self->{'diag_tree'}->set_cursor($path) if($ok);
   }
 }
 
 sub passe_suivant {
   my ($self)=@_;
+  my $nav=$self->{navigate}->get_active();
   my ($path)=$self->{'diag_tree'}->get_cursor();
+  my $n=$self->{diag_store}->iter_n_children(undef);
   if($path) {
-    my $path_next=Gtk2::TreePath->new ($path->to_string);
-    $path_next->next();
-    $self->{'diag_tree'}->set_cursor($path_next);
-    ($path_next)=$self->{'diag_tree'}->get_cursor();
-    $self->{'diag_tree'}->set_cursor($path)
-      if(!$path_next);
+    do {
+      $path->next();
+    } while($path->to_string<$n && !$self->navigate_ok($nav,$path));
+    $self->{'diag_tree'}->set_cursor($path) if($path->to_string<$n);
   }
 }
 
@@ -641,7 +688,7 @@ sub duplique_saisie {
 
    $self->{'capture'}->begin_transaction;
    $copy=$self->{'capture'}->new_page_copy($student,$page);
-   $self->{'capture'}->variable('annotate_source_change',time());
+   annotate_source_change($self->{'capture'});
    $self->{'capture'}->set_page_manual($student,$page,$copy,-1);
    $self->{'capture'}->end_transaction;
 
@@ -687,4 +734,3 @@ sub goto_activate_cb {
 1;
 
 __END__
-
