@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2015 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -24,12 +24,18 @@ use File::Spec::Functions qw/tmpdir/;
 
 use AMC::Basic;
 use AMC::DataModule::capture ':zone';
+use AMC::Gui::Prefs;
 
 use Gtk2 -init;
 
 use POSIX qw(ceil);
 
-use constant ID_AMC_BOX => 100;
+use constant {
+  ID_AMC_BOX => 100,
+
+  ZOOMS_EDIT_DND => 0,
+  ZOOMS_EDIT_CLICK => 1,
+};
 
 my $col_manuel = Gtk2::Gdk::Color->new(223*256,224*256,133*256);
 my $col_modif = Gtk2::Gdk::Color->new(226*256,184*256,178*256);
@@ -41,6 +47,7 @@ sub new {
 	'n_cols'=>4,
 	'factor'=>0.75,
 	'seuil'=>0.15,
+	'seuil_up'=>1.0,
 	'prop_min'=>0.30,
 	'prop_max'=>0.60,
 	'global'=>0,
@@ -51,12 +58,26 @@ sub new {
 	'size-prefs'=>'',
 	'encodage_interne'=>'UTF-8',
 	'list_view'=>'',
+	'zooms_edit_mode'=>ZOOMS_EDIT_DND,
+	'global_options'=>'',
+	'prefs'=>'',
     };
 
     for (keys %o) {
 	$self->{$_}=$o{$_} if(defined($self->{$_}));
     }
 
+    if($self->{global_options}) {
+      $self->{zooms_edit_mode}=$self->{global_options}->{zooms_edit_mode};
+    }
+
+    $self->{prefs}->store_register(
+# TRANSLATORS: One of the ways to change a box's ticked state in the zooms window
+				   'zooms_edit_mode'=>cb_model(ZOOMS_EDIT_DND,__"drag and drop",
+# TRANSLATORS: One of the ways to change a box's ticked state in the zooms window
+							       ZOOMS_EDIT_CLICK,__"click"),
+				  );
+
     $self->{'ids'}=[];
     $self->{'pb_src'}={};
     $self->{'real_src'}={};
@@ -93,10 +114,12 @@ sub new {
     $self->{'gui'}->set_translation_domain('auto-multiple-choice');
     $self->{'gui'}->add_from_file($glade_xml);
 
-    for(qw/main_window zooms_table_0 zooms_table_1 decoupage scrolled_0 scrolled_1 label_0 label_1 event_0 event_1 button_apply button_close info button_previous button_next/) {
+    for(qw/main_window zooms_table_0 zooms_table_1 decoupage scrolled_0 scrolled_1 label_0 label_1 event_0 event_1 button_apply button_close info button_previous button_next /) {
 	$self->{$_}=$self->{'gui'}->get_object($_);
     }
 
+    $self->{prefs}->transmet_pref($self->{'gui'},'zooms',$self);
+
     $self->{'label_0'}->set_markup('<b>'.$self->{'label_0'}->get_text.'</b>');
     $self->{'label_1'}->set_markup('<b>'.$self->{'label_1'}->get_text.'</b>');
     $self->{'info'}->set_markup('<b>'.sprintf(__("Boxes zooms for page %s"),
@@ -129,6 +152,24 @@ sub new {
     return($self);
 }
 
+sub edit_mode_update {
+  my ($self)=@_;
+
+  $self->{prefs}->reprend_pref('zooms',$self);
+  if($self->{global_options}) {
+    if($self->{global_options}->{zooms_edit_mode} != $self->{zooms_edit_mode}) {
+      $self->{global_options}->{_modifie_ok}=1;
+      $self->{global_options}->{zooms_edit_mode}=$self->{zooms_edit_mode};
+    }
+  }
+}
+
+sub dnd_mode { # drag and drop
+  my ($self)=@_;
+
+  return($self->{zooms_edit_mode} == ZOOMS_EDIT_DND);
+}
+
 sub clear_boxes {
     my ($self)=@_;
 
@@ -166,7 +207,8 @@ sub affect_box {
   my ($self,$z)=@_;
 
   my $id=$z->{'id_a'}.'-'.$z->{'id_b'};
-  my $auto_pos=($z->{'black'} > $z->{'total'}*$self->{'seuil'} ? 1 : 0);
+  my $auto_pos=($z->{'black'} >= $z->{'total'}*$self->{'seuil'} &&
+		$z->{'black'} <= $z->{'total'}*$self->{'seuil_up'} ? 1 : 0);
   my $eff_pos=($self->{'page_info'}->{'timestamp_manual'}
 	       && $z->{'manual'}>=0 ? $z->{'manual'} :
 	       $auto_pos);
@@ -236,6 +278,19 @@ sub safe_pixbuf {
   return($p,0);
 }
 
+sub buttons_availability {
+  my ($self)=@_;
+  if($self->{'conforme'}) {
+    $self->{'button_apply'}->hide();
+    $self->{'button_previous'}->set_sensitive($self->list_prev ? 1 : 0);
+    $self->{'button_next'}->set_sensitive($self->list_next ? 1 : 0);
+  } else {
+    $self->{'button_apply'}->show();
+    $self->{'button_previous'}->set_sensitive(0);
+    $self->{'button_next'}->set_sensitive(0);
+  }
+}
+
 sub load_boxes {
     my ($self)=@_;
 
@@ -285,10 +340,23 @@ sub load_boxes {
 			   $id );
 	$self->{'eb'}->{$id}
 	  ->signal_connect('drag-begin'=>sub {
-			     $self->{'eb'}->{$id}
-			       ->drag_source_set_icon_pixbuf($self->{'image'}->{$id}->get_pixbuf);
+			     if($self->dnd_mode) {
+			       $self->{'eb'}->{$id}
+				 ->drag_source_set_icon_pixbuf($self->{'image'}->{$id}->get_pixbuf);
+			     }
 			   });
 
+	$self->{'eb'}->{$id}
+	  ->signal_connect('button_press_event'=>sub {
+			     $self->{'button_event'}=$id; });
+	$self->{'eb'}->{$id}
+	  ->signal_connect('leave_notify_event'=>sub {
+			     $self->{'button_event'}=''; });
+	$self->{'eb'}->{$id}
+	  ->signal_connect('button_release_event'=>sub {
+			     my ($w,$event)=@_;
+			     $self->click_action($id,$event); });
+
 	$self->{'position'}->{$id}=$self->{'eff_pos'}->{$id};
 
 	push @ids,$id;
@@ -319,12 +387,7 @@ sub load_boxes {
     $va=$self->{'scrolled_1'}->get_vadjustment();
     $va->clamp_page($va->lower(),$va->lower());
 
-    if($self->{'conforme'}) {
-	$self->{'button_apply'}->hide();
-    } else {
-	$self->{'button_apply'}->show();
-    }
-
+    $self->buttons_availability;
 }
 
 sub refill {
@@ -332,11 +395,7 @@ sub refill {
     $self->{'conforme'}=1;
     for(0,1) { $self->vide($_); }
     for(0,1) { $self->remplit($_); }
-    if($self->{'conforme'}) {
-	$self->{'button_apply'}->hide();
-    } else {
-	$self->{'button_apply'}->show();
-    }
+    $self->buttons_availability;
     $self->scrolled_update;
 }
 
@@ -365,6 +424,36 @@ sub page {
     $self->load_boxes;
 }
 
+sub click_action {
+  my ($self,$id,$event)=@_;
+  if((!$self->dnd_mode) && ($self->{'button_event'} eq $id)) {
+    if($event->button==1) {
+      $self->toggle($id);
+    } elsif($event->button==3) {
+      my $cat=$self->{'position'}->{$id};
+      my @toggle_ids=();
+      my $t=$cat;
+      for my $i (@{$self->{'ids'}}) {
+	$t=1 if($cat==0 && $id eq $i);
+	if($t && $self->{'position'}->{$i} == $cat) {
+	  push @toggle_ids,$i;
+	}
+	$t=0 if($cat==1 && $id eq $i);
+      }
+      for my $i (@toggle_ids) {
+	$self->{'position'}->{$i}=1-$cat;
+      }
+      $self->refill;
+    }
+  }
+}
+
+sub toggle {
+  my ($self,$id)=@_;
+  $self->{'position'}->{$id}=1-$self->{'position'}->{$id};
+  $self->refill;
+}
+
 sub source_drag_data_get {
     my ($widget, $context, $data, $info, $time,$string) = @_;
     $data->set_text($string,-1);
@@ -374,11 +463,15 @@ sub target_drag_data_received {
     my ($widget, $context, $x, $y, $data, $info, $time,$args) = @_;
     my ($self,$cat)=@$args;
     my $id=$data->get_text();
-    debug "Page ".pageids_string(@{$self->{'page_id'}})
-      .": move $id to category $cat\n";
-    if($self->{'position'}->{$id} != $cat) {
+    if($self->dnd_mode()) {
+      debug "Page ".pageids_string(@{$self->{'page_id'}})
+	.": move $id to category $cat\n";
+      if($self->{'position'}->{$id} != $cat) {
 	$self->{'position'}->{$id}=$cat;
 	$self->refill;
+      }
+    } else {
+      debug "Drang and drop cancelled: CLICK mode";
     }
 }
 
@@ -491,26 +584,41 @@ sub zoom_arriere {
     $self->zoom_it();
 }
 
-sub zoom_list_previous {
+sub list_prev {
   my ($self)=@_;
   my ($path)=$self->{'list_view'}->get_cursor();
   if($path) {
     if($path->prev) {
-      $self->{'list_view'}->set_cursor($path);
+      return($path);
     }
   }
 }
 
+sub list_next {
+  my ($self)=@_;
+  my ($path)=$self->{'list_view'}->get_cursor();
+  if($path) {
+    $path->next;
+    my $next_iter=$self->{'list_view'}->get_model->get_iter($path);
+    if($next_iter) {
+      return($self->{'list_view'}->get_model->get_path($next_iter));
+    }
+  }
+}
+
+sub zoom_list_previous {
+  my ($self)=@_;
+  my $path_prev=$self->list_prev;
+  if($path_prev) {
+    $self->{'list_view'}->set_cursor($path_prev);
+  }
+}
+
 sub zoom_list_next {
   my ($self)=@_;
-  my ($path)=$self->{'list_view'}->get_cursor();
-  if($path) {
-    my $path_next=Gtk2::TreePath->new ($path->to_string);
-    $path_next->next();
+  my $path_next=$self->list_next;
+  if($path_next) {
     $self->{'list_view'}->set_cursor($path_next);
-    ($path_next)=$self->{'list_view'}->get_cursor();
-    $self->{'list_view'}->set_cursor($path)
-      if(!$path_next);
   }
 }
 
