@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2015 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -29,6 +29,7 @@ sub new {
 
     my $self={'onerror'=>'stderr',
 	      'seuil'=>0,
+	      'seuil_up'=>1.0,
 	      'data'=>'',
 	      'default_strategy'=>'',
 	      'default_strategy_plain'=>'',
@@ -70,7 +71,8 @@ sub error {
 sub ticked {
   my ($self,$student,$copy,$question,$answer)=@_;
   return($self->{'_capture'}
-	 ->ticked($student,$copy,$question,$answer,$self->{'seuil'}));
+	 ->ticked($student,$copy,$question,$answer,
+		  $self->{'seuil'},$self->{'seuil_up'}));
 }
 
 # tells if the answer given by the student is the correct one (ticked
@@ -254,7 +256,8 @@ sub syntax_error {
 # computed from this formula
 sub use_formula {
   my ($self,$score,$why)=@_;
-  if($self->defined_directive("formula")) {
+  if($self->defined_directive("formula")
+    && $self->directive("formula") =~ /[^\s]/) {
     # a formula is given to compute the score directly
     debug "Using formula";
     $$score=$self->directive("formula");
@@ -372,6 +375,10 @@ sub score_question {
     debug "Scoring syntax error: $err";
     $xx=$self->directive("e");
     $why='E';
+  } elsif($self->variable("INVALID")) {
+    debug "INVALID variable is set";
+    $xx=$self->directive("e");
+    $why='E';
   }
 
   if(!$why) {
