@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2008-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -36,6 +36,22 @@ use AMC::DataModule::scoring ':question';
 use_gettext;
 use_amc_plugins();
 
+my $cmd_pid='';
+my @output_files=();
+
+sub catch_signal {
+    my $signame = shift;
+    debug "*** AMC-prepare : signal $signame, transfered to $cmd_pid...";
+    kill 9,$cmd_pid if($cmd_pid);
+    if(@output_files) {
+      debug "Removing files that are beeing built: ".join(" ",@output_files);
+      unlink(@output_files);
+    }
+    die "Killed";
+}
+
+$SIG{INT} = \&catch_signal;
+
 # PARAMETERS
 
 my $mode="mbs";
@@ -61,12 +77,15 @@ my $progress_id='';
 my $out_calage='';
 my $out_sujet='';
 my $out_corrige='';
+my $out_corrige_indiv='';
 my $out_catalog='';
 
 my $jobname="amc-compiled";
 
 my $f_tex;
 
+@ARGV=unpack_args(@ARGV);
+
 GetOptions("mode=s"=>\$mode,
 	   "with=s"=>\$latex_engine,
 	   "data=s"=>\$data_dir,
@@ -74,6 +93,7 @@ GetOptions("mode=s"=>\$mode,
 	   "out-calage=s"=>\$out_calage,
 	   "out-sujet=s"=>\$out_sujet,
 	   "out-corrige=s"=>\$out_corrige,
+	   "out-corrige-indiv=s"=>\$out_corrige_indiv,
 	   "out-catalog=s"=>\$out_catalog,
 	   "convert-opts=s"=>\$convert_opts,
 	   "debug=s"=>\$debug,
@@ -129,6 +149,23 @@ sub split_latex_engine {
 
 split_latex_engine();
 
+sub set_filtered_source {
+  my ($filtered_source)=@_;
+
+  # change directory where the $filtered_source is, and set $f_base to
+  # the $filtered_source without path and without extension
+
+  ($v,$d,$f_tex)=splitpath($filtered_source);
+  chdir(catpath($v,$d,""));
+  $f_base=$f_tex;
+  $f_base =~ s/\.tex$//i;
+
+  # AMC usualy sets $prefix to "DOC-", but if $prefix is empty, uses
+  # the base name
+
+  $prefix=$f_base."-" if(!$prefix);
+}
+
 # Uses an AMC::Gui::Avancement object to tell regularly the calling
 # program how much work we have done so far.
 
@@ -161,12 +198,21 @@ for(\$data_dir,\$source,\$filtered_source) {
     $$_=rel2abs($$_);
 }
 
+set_filtered_source($filtered_source);
+
 # These variables are used to track errors from LaTeX compiling
 
 my $a_errors; # the number of errors
 my @errors_msg=(); # errors messages (questions specifications problems)
 my @latex_errors=(); # LaTeX compilation errors
 
+sub flush_errors {
+  debug(@errors_msg);
+  print join('',@errors_msg);
+  @errors_msg=();
+}
+
+
 # %info_vars collects the variables values that LaTeX wants to give us
 
 my %info_vars=();
@@ -294,7 +340,7 @@ sub analyse_amclog {
     # refers to ID (question name, string given as an argument to
     # question environment)
 
-    if (/AUTOQCM\[NUM=([0-9]+)=([^\]]+)\]/) {
+    if (/AUTOQCM\[NUM=([0-9]+)=(.+)\]/) {
       # stores this association (two-way)
 
       $titres{$1}=$2;
@@ -356,8 +402,7 @@ sub analyse_amclog {
 
   # Send error messages to the calling program through STDOUT
 
-  debug(@errors_msg);
-  print join('',@errors_msg);
+  flush_errors();
 
   debug("AMC log $amclog_file : $a_errors errors.");
 }
@@ -366,15 +411,28 @@ sub analyse_amclog {
 # again if needed (for exemple when a second run is necessary to get
 # references right), and then produces a PDF file from LaTeX output.
 #
-# $oo{command} should be the LaTeX command to run, with all necessary
-# arguments
+# $oo{command_opts} should be the options to be passed to latex_cmd, to
+# build the LaTeX command to run, with all necessary arguments
+
+my $filter_engine;
 
 sub execute {
     my %oo=(@_);
+    my $errs=0;
 
-    my $n_run=0; # number of runs so far
-    my $rerun=0; # has to re-run?
-    my $format=''; # output format
+    prepare_filter();
+
+    # gives the processing command to the filter
+    $oo{command}=[latex_cmd(@{$oo{command_opts}})];
+    $ENV{AMC_CMD}=join(' ',@{$oo{command}});
+
+    if($filter) {
+      if(!$filter_engine->get_filter_result('done')
+	 || $filter_engine->get_filter_result('jobspecific')) {
+	$errs=do_filter();
+	$filter_engine->set_filter_result('done',1) if(!$errs);
+      }
+    }
 
     # first removes previous run's outputs
 
@@ -385,6 +443,20 @@ sub execute {
 	}
     }
 
+    exit 1 if($errs);
+
+    # the filter could have changed the latex engine, so update it
+    $oo{command}=[latex_cmd(@{$oo{command_opts}})];
+    $ENV{AMC_CMD}=join(' ',@{$oo{command}});
+
+    check_engine();
+
+    my $min_runs=1; # minimum number of runs
+    my $max_runs=2; # maximum number of runs
+    my $n_run=0; # number of runs so far
+    my $rerun=0; # has to re-run?
+    my $format=''; # output format
+
     do {
 
 	$n_run++;
@@ -407,7 +479,10 @@ sub execute {
 	    # LaTeX Warning: Label(s) may have changed. Rerun to get
 	    # cross-references right. -> has to re-run
 
-	    $rerun=1 if(/^LaTeX Warning:.*Rerun to get cross-references right/);
+	    $rerun=1
+	      if(/^LaTeX Warning:.*Rerun to get cross-references right/);
+	    $min_runs=2
+	      if(/Warning: .*run twice/);
 
 	    # Output written on jobname.pdf (10 pages) -> output
 	    # format is pdf
@@ -430,7 +505,7 @@ sub execute {
 	close(EXEC);
 	$cmd_pid='';
 
-    } while($rerun && $n_run<=1 && ! $oo{'once'});
+    } while( (($n_run<$min_runs) || ($rerun && $n_run<$max_runs)) && ! $oo{'once'});
 
     # For these engines, we already know what is the output format:
     # override detected one
@@ -489,52 +564,53 @@ sub execute {
 # do_filter() converts the source file to LaTeX format, using the
 # right AMC::Filter::* module
 
+sub prepare_filter {
+  if($filter) {
+    if(!$filter_engine) {
+      load("AMC::Filter::$filter");
+      $filter_engine="AMC::Filter::$filter"->new(jobname=>$jobname);
+      $filter_engine->pre_filter($source);
+
+      # sometimes the filter says that the source file don't need to
+      # be changed
+
+      set_filtered_source($source)
+	if($filter_engine->unchanged);
+    }
+  } else {
+    # Empty filter: the source is already a LaTeX file
+    set_filtered_source($source);
+  }
+}
+
 sub do_filter {
   my $f_base;
   my $v;
   my $d;
+  my $n_err=0;
 
   if($filter) {
     # Loads and call appropriate filter to convert $source to
     # $filtered_source
 
-    load("AMC::Filter::$filter");
-    my $filter="AMC::Filter::$filter"->new();
-    $filter->filter($source,$filtered_source);
+    prepare_filter();
+    $filter_engine->filter($source,$filtered_source);
 
     # show conversion errors
 
-    for($filter->errors()) {
+    for($filter_engine->errors()) {
       print "ERR: $_\n";
+      $n_err++;
     }
 
     # sometimes the filter asks to override the LaTeX engine
 
-    split_latex_engine($filter->{'project_options'}->{'moteur_latex_b'})
-      if($filter->{'project_options'}->{'moteur_latex_b'});
+    split_latex_engine($filter_engine->{'project_options'}->{'moteur_latex_b'})
+      if($filter_engine->{'project_options'}->{'moteur_latex_b'});
 
-    # and sometimes the filter says that the source file don't need to
-    # be changed
-
-    $filtered_source=$source
-      if($filter->unchanged);
-  } else {
-    # Empty filter: the source is already a LaTeX file
-    $filtered_source=$source;
   }
 
-  # change directory where the $filtered_source is, and set $f_base to
-  # the $filtered_source without path and without extension
-
-  ($v,$d,$f_tex)=splitpath($filtered_source);
-  chdir(catpath($v,$d,""));
-  $f_base=$f_tex;
-  $f_base =~ s/\.tex$//i;
-
-  # AMC usualy sets $prefix to "DOC-", but if $prefix is empty, uses
-  # the base name
-
-  $prefix=$f_base."-" if(!$prefix);
+  return($n_err);
 }
 
 # give_latex_errors($context) Relay suitably formatted LaTeX errors to
@@ -614,22 +690,6 @@ if($to_do{f}) {
   do_filter();
 }
 
-############################################################################
-# MODE k: builds individual corrected answer sheets (exactly the same
-# sheets as for the students, but with correct answers ticked).
-############################################################################
-
-if($to_do{k}) {
-
-    do_filter();
-
-    check_engine();
-
-    execute('command'=>[latex_cmd(qw/NoWatermarkExterne 1 NoHyperRef 1 CorrigeIndivExterne 1/)]);
-    transfer("$jobname.pdf",($out_corrige ? $out_corrige : $prefix."corrige.pdf"));
-    give_latex_errors(__"individual solution");
-}
-
 ############################################################################
 # MODE s: builds the subject and a solution (with all the answers for
 # questions, but with a different layout)
@@ -638,9 +698,7 @@ if($to_do{k}) {
 if($to_do{s}) {
   $to_do{s}='[sc]' if($to_do{s} eq '1');
 
-    do_filter();
-
-    check_engine();
+  @output_files=($out_sujet,$out_calage,$out_corrige,$out_catalog);
 
     my %opts=(qw/NoWatermarkExterne 1 NoHyperRef 1/);
 
@@ -649,7 +707,7 @@ if($to_do{s}) {
     $out_catalog=$prefix."catalog.pdf" if(!$out_catalog);
     $out_sujet=$prefix."sujet.pdf" if(!$out_sujet);
 
-    for my $f ($out_calage,$out_corrige,$out_sujet) {
+    for my $f ($out_calage,$out_corrige,$out_corrige_indiv,$out_sujet,$out_catalog) {
 	if(-f $f) {
 	    debug "Removing already existing file: $f";
 	    unlink($f);
@@ -658,7 +716,7 @@ if($to_do{s}) {
 
     # 1) SUBJECT
 
-    execute('command'=>[latex_cmd(%opts,'SujetExterne'=>1)]);
+    execute('command_opts'=>[%opts,'SujetExterne'=>1]);
     analyse_amclog("$jobname.amc");
     give_latex_errors(__"question sheet");
 
@@ -667,6 +725,31 @@ if($to_do{s}) {
     transfer("$jobname.pdf",$out_sujet);
     transfer("$jobname.xy",$out_calage);
 
+  # Looks for accents problems in question IDs...
+
+  my %qids=();
+  my $unknown_qid=0;
+  if(open(XYFILE,$out_calage)) {
+    binmode(XYFILE);
+    while(<XYFILE>) {
+      if(!utf8::decode($_) || /\\IeC/) {
+	if(/\\tracepos\{[^:]*:[^:]*:(.+):[^:]*\}\{([+-]?[0-9.]+[a-z]*)\}\{([+-]?[0-9.]+[a-z]*)\}(?:\{([a-zA-Z]*)\})?$/) {
+	  $qids{$1}=1;
+	} else {
+	  $unknown_qid=1;
+	}
+      }
+    }
+    close(XYFILE);
+    if(%qids) {
+      push @errors_msg,
+	map { "WARN: ".sprintf(__("please remove accentuated or non-standard characters from the following question ID: \"%s\""),$_)."\n" } (sort { $a cmp $b } (keys %qids));
+    } elsif($unknown_qid) {
+      push @errors_msg,"WARN: ".__("some question IDs seems to have accentuated or non-standard characters. This may break future processings.")."\n";
+    }
+  }
+  flush_errors();
+
     # Relays variables to calling process
 
     print "Variables :\n";
@@ -677,7 +760,7 @@ if($to_do{s}) {
     # 2) SOLUTION
 
   if($to_do{s}=~/s/) {
-    execute('command'=>[latex_cmd(%opts,'CorrigeExterne'=>1)]);
+    execute('command_opts'=>[%opts,'CorrigeExterne'=>1]);
     transfer("$jobname.pdf",$out_corrige);
     give_latex_errors(__"solution");
   } else {
@@ -688,7 +771,7 @@ if($to_do{s}) {
     # 3) CATALOG
 
   if($to_do{s}=~/c/) {
-    execute('command'=>[latex_cmd(%opts,'CatalogExterne'=>1)]);
+    execute('command_opts'=>[%opts,'CatalogExterne'=>1]);
     transfer("$jobname.pdf",$out_catalog);
     give_latex_errors(__"catalog");
   } else {
@@ -697,6 +780,29 @@ if($to_do{s}) {
   }
 }
 
+############################################################################
+# MODE k: builds individual corrected answer sheets (exactly the same
+# sheets as for the students, but with correct answers ticked).
+############################################################################
+
+if($to_do{k}) {
+
+  my $of=$out_corrige_indiv;
+  $of=$out_corrige if(!$of && !$to_do{s});
+  $of=$prefix."corrige.pdf" if(!$of);
+
+  if(-f $of) {
+    debug "Removing already existing file: $of";
+    unlink($of);
+  }
+
+  @output_files=($of);
+
+  execute('command_opts'=>[qw/NoWatermarkExterne 1 NoHyperRef 1 CorrigeIndivExterne 1/]);
+  transfer("$jobname.pdf",$of);
+  give_latex_errors(__"individual solution");
+}
+
 ############################################################################
 # MODE b: extracts the scoring strategy to the scoring database,
 # parsing the AUTOQCM[...] messages from the LaTeX output.
@@ -706,10 +812,6 @@ if($to_do{b}) {
 
     print "********** Making marks scale...\n";
 
-    do_filter();
-
-    check_engine();
-
     my %bs=();
     my %titres=();
 
@@ -720,6 +822,13 @@ if($to_do{b}) {
 
     my $delta=0;
 
+    # Launches the LaTeX engine
+
+    execute('command_opts'=>[qw/ScoringExterne 1 NoHyperRef 1/],
+	    'once'=>1);
+
+    open(AMCLOG,"$jobname.amc") or die "Unable to open $jobname.amc : $!";
+
     # Opens a connection with the database
 
     my $data=AMC::Data->new($data_dir);
@@ -729,18 +838,12 @@ if($to_do{b}) {
     my $qs={};
     my $current_q={};
 
-    $scoring->begin_transaction('ScEx');
-    $capture->variable('annotate_source_change',time());
-    $scoring->clear_strategy;
-
-    # Launches the LaTeX engine
-
-    execute('command'=>[latex_cmd(qw/CalibrationExterne 1 NoHyperRef 1/)],
-	    'once'=>1);
-    open(AMCLOG,"$jobname.amc") or die "Unable to open $jobname.amc : $!";
-
     # and parse the log...
 
+    $scoring->begin_transaction('ScEx');
+    annotate_source_change($capture);
+    $scoring->clear_strategy;
+
     while(<AMCLOG>) {
 	debug($_) if($_);
 
@@ -794,7 +897,7 @@ if($to_do{b}) {
 	  $qs={};
 	}
 
-	if(/AUTOQCM\[NUM=([0-9]+)=([^\]]+)\]/) {
+	if(/AUTOQCM\[NUM=([0-9]+)=(.+)\]/) {
 	  # association question-number<->question-title
 	  $scoring->question_title($1,$2);
 	}
@@ -824,7 +927,9 @@ if($to_do{b}) {
 	# AUTOQCM[BR=N] tells that this student is a replicate of student N
 
 	if(/AUTOQCM\[BR=([0-9]+)\]/) {
-	  $scoring->replicate($1,$etu);
+	  my $alias=$1;
+	  $scoring->replicate($alias,$etu);
+	  $etu=$alias;
 	}
 
 	if(/AUTOQCM\[B=([^\]]+)\]/) {
@@ -865,7 +970,6 @@ if($to_do{b}) {
 	}
     }
     close(AMCLOG);
-    $cmd_pid='';
 
     $scoring->end_transaction('ScEx');
 }
