@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2008-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -99,6 +99,10 @@ GetOptions("projet=s"=>\$projet_dir,
 
 set_debug($debug);
 
+print (("*"x60)."\n");
+print "* WARNING: AMC-regroupe is now obsolete\n* Please move to AMC-annotate\n";
+print (("*"x60)."\n");
+
 $projet_dir =~ s:/+$::;
 
 $temp_dir = tempdir( DIR=>tmpdir(),
@@ -474,7 +478,7 @@ for my $e (@students) {
 
       if($i) {
 	debug "Name found";
-	($n)=$noms->data($lk,$i);
+	($n)=$noms->data($lk,$i,test_numeric=>1);
 	if($n) {
 	  $f=$noms->substitute($n,$f);
 	}
