@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2008-2010,2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -26,7 +26,9 @@ use Archive::Tar;
 my $liste='';
 
 GetOptions("liste=s"=>\$liste,
-	   );
+	  );
+
+my $tar_opts={uid=>0,gid=>0,uname=>'root',gname=>'root',mtime=>1420066800};
 
 my @fichiers=@ARGV;
 
@@ -80,15 +82,17 @@ for my $f (@fichiers) {
 
 	    my $tar = Archive::Tar->new;
 
-	    $tar->add_data("$code_name.$ext",encode_utf8($ex));
+	    $tar->add_data("$code_name.$ext",encode_utf8($ex),$tar_opts);
+            $tar->chmod("$code_name.$ext",'0644');
 	    $tar->add_data("description.xml",
 			   encode_utf8('<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <description>
   <title>'.$name.'</title>
   <text>'.$desc.'</text>
 </description>
-')
-			   );
+'),$tar_opts
+                          );
+            $tar->chmod("description.xml",'0644');
 	    my $opts='<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <projetAMC>
   <texsrc>%PROJET/'.$code_name.'.'.$ext.'</texsrc>
@@ -103,8 +107,8 @@ for my $f (@fichiers) {
 	    $opts .= '</projetAMC>
 ';
 	    $tar->add_data("options.xml",
-			   encode_utf8($opts));
-
+			   encode_utf8($opts),$tar_opts);
+            $tar->chmod("options.xml",'0644');
 	    $tar->write("$rep/$code_name.tgz", COMPRESS_GZIP);
 
 	    print LOG "$rep/$code_name.tgz\n" if($liste);
