@@ -1,6 +1,6 @@
 #! /usr/bin/perl -w
 #
-# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2016 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -25,11 +25,11 @@ use AMC::Data;
 use AMC::DataModule::capture qw(:zone);
 use AMC::DataModule::scoring qw(:question);
 use AMC::Scoring;
+use AMC::NamesFile;
 
 use Text::CSV;
 use File::Spec::Functions qw(tmpdir);
 use File::Temp qw(tempfile tempdir);
-use File::Copy::Recursive qw(rcopy);
 use File::Copy;
 use Digest::MD5;
 
@@ -61,6 +61,7 @@ sub new {
      'debug_pixels'=>0,
      'scans'=>'',
      'seuil'=>0.5,
+     'seuil_up'=>1.0,
      'bw_threshold'=>0.6,
      'tol_marque'=>0.4,
      'rounding'=>'i',
@@ -72,6 +73,7 @@ sub new {
      'list_key'=>'id',
      'code'=>'student',
      'check_assoc'=>'',
+     'association_manual'=>'',
      'annote'=>'',
      'annote_files'=>[],
      'annote_ascii'=>0,
@@ -84,10 +86,13 @@ sub new {
      'to_check'=>[],
      'export_full_csv'=>[],
      'export_csv_ticked'=>'AB',
+     'export_ods'=>'',
      'blind'=>0,
      'check_zooms'=>{},
      'skip_prepare'=>0,
      'skip_scans'=>0,
+     'tracedest'=>'STDERR',
+     'debug_file'=>'',
     };
 
   for (keys %oo) {
@@ -101,7 +106,7 @@ sub new {
   if (!$self->{'src'}) {
     opendir(my $dh, $self->{'dir'})
       || die "can't opendir $self->{'dir'}: $!";
-    my @tex = grep { /\.(tex|txt)$/ } readdir($dh);
+    my @tex = grep { /\.(tex|txt)$/ } sort { $a cmp $b } readdir($dh);
     closedir $dh;
     $self->{'src'}=$tex[0];
   }
@@ -113,8 +118,18 @@ sub new {
     closedir $dh;
     $self->{'list'}=$l[0];
   }
+  $self->{names}=AMC::NamesFile::new($self->{'dir'}.'/'.$self->{list},'utf8','id')
+    if(-f $self->{'dir'}.'/'.$self->{list});
 
-  GetOptions("debug!"=>\$self->{'debug'},"blind!"=>\$self->{'blind'});
+  my $to_stdout=0;
+
+  GetOptions("debug!"=>\$self->{'debug'},
+             "blind!"=>\$self->{'blind'},
+             "log-to=s"=>\$self->{debug_file},
+             "to-stdout!"=>\$to_stdout);
+
+  $self->{tracedest} = 'STDOUT' if($to_stdout);
+  binmode $self->{tracedest}, ":utf8";
 
   $self->install;
 
@@ -152,9 +167,14 @@ sub install {
   $self->{'temp_dir'} = tempdir( DIR=>$temp_loc,
 				 CLEANUP => (!$self->{'debug'}) );
 
-  rcopy($self->{'dir'}.'/*',$self->{'temp_dir'});
+  opendir(my $sh,$self->{'dir'})
+    || die "can't opendir $self->{dir}: $!";
+  for my $f (grep { ! /^\./ } (readdir($sh))) {
+    system("cp","-r",$self->{'dir'}.'/'.$f,$self->{'temp_dir'});
+  }
+  closedir $sh;
 
-  print STDERR "[>] Installed in $self->{'temp_dir'}\n";
+  print { $self->{tracedest} } "[>] Installed in $self->{'temp_dir'}\n";
 
   if(-d ($self->{'temp_dir'}."/scans") && !$self->{'scans'}) {
     opendir(my $dh, $self->{'temp_dir'}."/scans")
@@ -172,7 +192,8 @@ sub install {
     mkdir($self->{'temp_dir'}."/$d") if(!-d $self->{'temp_dir'}."/$d");
   }
 
-  $self->{'debug_file'}=$self->{'temp_dir'}."/debug.log";
+  $self->{'debug_file'}=$self->{'temp_dir'}."/debug.log"
+    if(!$self->{'debug_file'});
   open(DB,">",$self->{'debug_file'});
   print DB "Test\n";
   close(DB);
@@ -208,23 +229,34 @@ sub see_file {
   my $ff=$file;
   $ff =~ s:.*/::;
   if($self->{'ok_checksums'}->{$dig}) {
-    $self->trace("[T] File ok: $ff");
+    $self->trace("[T] File ok (checksum): $ff");
   } else {
-    my $i=0;
-    my $dest;
-    do {
-      $dest=sprintf("%s/%04d-%s",$self->{'check_dir'},$i,$ff);
-      $i++;
-    } while(-f $dest);
-    copy($file,$dest);
-    push @{$self->{'to_check'}},[$dig,$dest];
+    # compares with already validated file
+    my $validated=$self->{'temp_dir'}."/checked/$ff";
+    if(-f $validated && $ff =~ /\.pdf$/i) {
+      if(run('comparepdf','-ca','-v0',$validated,$file)) {
+	$self->trace("[T] File ok (compare): $ff");
+      } else {
+	$self->trace("[E] File different (compare): $ff");
+	exit(1);
+      }
+    } else {
+      my $i=0;
+      my $dest;
+      do {
+	$dest=sprintf("%s/%04d-%s",$self->{'check_dir'},$i,$ff);
+	$i++;
+      } while(-f $dest);
+      copy($file,$dest);
+      push @{$self->{'to_check'}},[$dig,$dest];
+    }
   }
 }
 
 sub trace {
   my ($self,@m)=@_;
-  print STDERR join(' ',@m)."\n";
-  open LOG,">>$self->{'debug_file'}";
+  print { $self->{tracedest} } join(' ',@m)."\n";
+  open LOG,">>:utf8",$self->{'debug_file'};
   print LOG join(' ',@m)."\n";
   close LOG;
 }
@@ -352,6 +384,7 @@ sub note {
   $self->amc_command('note',
 		     '--data','%DATA',
 		     '--seuil',$self->{'seuil'},
+		     '--seuil-up',$self->{'seuil_up'},
 		     '--grain',$self->{'grain'},
 		     '--arrondi',$self->{'rounding'},
 		     '--notemax',$self->{'notemax'},
@@ -377,7 +410,20 @@ sub assoc {
 		     '--liste-key',$self->{'list_key'},
 		     @code,
 		     '--data','%DATA',
-		     );
+                    );
+
+  if($self->{association_manual}) {
+    for my $a (@{$self->{association_manual}}) {
+      $self->amc_command('association',
+                         '--liste','%PROJ/'.$self->{'list'},
+                         '--data','%DATA',
+                         '--set',
+                         '--student',$a->{student},
+                         '--copy',$a->{copy},
+                         '--id',$a->{id},
+                         );
+    }
+  }
 }
 
 sub get_marks {
@@ -465,7 +511,14 @@ sub get_assoc {
 
     $self->trace("[I] Assoc:");
     for my $m (@{$self->{'association'}}) {
-      $self->trace("    ".join(' ',map { $_."=".$m->{$_} } (qw/student copy auto manual/)));
+      for my $t (qw/auto manual/) {
+        my ($n)=$self->{names}->data('id',$m->{$t},test_numeric=>1);
+        if($n) {
+          $m->{$t}=$n->{id};
+          $m->{name}=$n->{_ID_};
+        }
+      }
+      $self->trace("    ".join(' ',map { $_."=".$m->{$_} } (qw/student copy auto manual name/)));
     }
   }
 }
@@ -515,30 +568,28 @@ sub annote {
   my ($self)=@_;
   return if(!$self->{'annote'});
 
-  $self->amc_command('annote',
-		     '--fich-noms','%PROJ/'.$self->{'list'},
-		     '--verdict',$self->{'verdict'},
-		     '--verdict-question',$self->{'verdict_question'},
-		     '--position',$self->{'annote_position'},
-		     '--projet','%PROJ',
-		     '--data','%DATA',
-		     );
-
   my $nf=$self->{'temp_dir'}."/num-pdf";
   open(NUMS,">$nf");
   for (@{$self->{'annote'}}) { print NUMS "$_\n"; }
   close(NUMS);
-  $self->amc_command('regroupe',
-		     ($self->{'annote_ascii'} ? "--force-ascii" : "--no-force-ascii"),
-		     '--projet','%PROJ',
-		     '--n-copies',$self->{'n_copies'},
-		     '--sujet','%PROJ/sujet.pdf',
+
+  $self->amc_command('annotate',
+		     '--names-file','%PROJ/'.$self->{'list'},
+		     '--verdict',$self->{'verdict'},
+		     '--verdict-question',$self->{'verdict_question'},
+		     '--position',$self->{'annote_position'},
+		     '--project','%PROJ',
 		     '--data','%DATA',
-		     '--tex-src','%PROJ/'.$self->{'src'},
+		     ($self->{'annote_ascii'}
+		      ? "--force-ascii" : "--no-force-ascii"),
+		     '--n-copies',$self->{'n_copies'},
+		     '--subject','%PROJ/sujet.pdf',
+		     '--src','%PROJ/'.$self->{'src'},
 		     '--with',$self->{'tex_engine'},
-		     '--modele',$self->{'model'},
+		     '--filename-model',$self->{'model'},
 		     '--id-file','%PROJ/num-pdf',
-		     '--fich-noms','%PROJ/'.$self->{'list'},
+		     '--darkness-threshold',$self->{'seuil'},
+		     '--darkness-threshold-up',$self->{'seuil_up'},
 		     );
 
   $pdf_dir=$self->{'temp_dir'}.'/cr/corrections/pdf';
@@ -641,6 +692,43 @@ sub check_export {
     }
     $self->end;
   }
+
+  if($self->{'export_ods'}) {
+    require OpenOffice::OODoc;
+
+    $self->begin("ODS full export test");
+    $self->amc_command('export',
+		       '--data','%DATA',
+		       '--module','ods',
+		       '--fich-noms','%PROJ/'.$self->{'list'},
+		       '--option-out','columns=student.copy',
+		       '--option-out','stats=h',
+		       '-o','%PROJ/export.ods',
+		      );
+    my $doc = OpenOffice::OODoc::odfDocument(file=>$self->{'temp_dir'}.'/export.ods');
+    my %iq=();
+    my $i=0;
+    while(my $id=$doc->getCellValue(1,0,$i)) {
+      $iq{$id}=$i;
+      $i+=5;
+    }
+  ONEQ: for my $q (@{$self->{'export_ods'}->{stats}}) {
+      my $i=$iq{$q->{id}};
+      if(defined($i)) {
+        $self->test($doc->getCellValue(1,2,$i+1),$q->{total},'total');
+        $self->test($doc->getCellValue(1,3,$i+1),$q->{empty},'empty');
+        $self->test($doc->getCellValue(1,4,$i+1),$q->{invalid},'invalid');
+        for my $a (@{$q->{answers}}) {
+          $self->test($doc->getCellValue(1,4+$a->{i},$i+1),$a->{ticked},
+                      'stats:'.$q->{id}.':'.$a->{i});
+        }
+      } else {
+        $self->trace("[E] Stats: question not found in stats table: $q->{id}");
+        exit 1;
+      }
+    }
+    $self->end;
+  }
 }
 
 sub check_zooms {
@@ -695,6 +783,9 @@ sub check_textest {
   }
   $tex_file=$self->{'temp_dir'}."/".$tex_file;
   if(-f $tex_file) {
+    my @value_is,@value_shouldbe;
+    chomp(my $cwd = `pwd`);
+    chdir($self->{'temp_dir'});
     open(TEX,"-|",$self->{'tex_engine'},
 	 $tex_file);
     while(<TEX>) {
@@ -709,8 +800,20 @@ sub check_textest {
       if(/^TEST\(([^,]*),([^,]*)\)/) {
 	$self->test($1,$2);
       }
+      if(/^VALUEIS\((.*)\)/) {
+        push @value_is,$1;
+      }
+      if(/^VALUESHOULDBE\((.*)\)/) {
+        push @value_shouldbe,$1;
+      }
     }
     close(TEX);
+    chdir($cwd);
+    if(@value_shouldbe) {
+      for my $i (0..$#value_shouldbe) {
+        $self->test($value_is[$i],$value_shouldbe[$i]);
+      }
+    }
     $self->end();
   } else {
     $self->trace("[X] TeX file not found: $tex_file");
@@ -809,11 +912,13 @@ sub test_scoring {
 			ZONE_BOX,1,0,$none_t);
   }
 
-  my $qdata=$s->student_scoring_base(1,0,0.5);
+  my $qdata=$s->student_scoring_base(1,0,0.5,1.0);
 
   $s->end_transaction('tSCO');
 
   my $scoring=AMC::Scoring->new(data=>$self->{'temp_dir'}."/data");
+  $scoring->set_default_strategy($question->{default_strategy})
+    if($question->{default_strategy});
 
   set_debug($self->{debug_file});
 
