@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2014 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -24,6 +24,8 @@ AMC::Test->new('dir'=>__FILE__,'filter'=>'plain','tex_engine'=>'xelatex',
 	       'export_full_csv'=>[
 				  {-copy=>3,-question=>'deuxdeux',-score=>1},
 				  {-copy=>3,-question=>'fusion',-score=>2},
+				  {-copy=>3,-question=>'france',-score=>3},
+				  {-copy=>3,-question=>'oceans',-score=>4},
 				  ],
 	      )->default_process;
 
